/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <UpTools/UpDnsBase.h>


DnsQueryType	dnsQueryType;

std::string		UpRR::notApplicable("not applicable");
UpAddress		UpRR::emptyAddr;

std::string dnsQueryStatus[] = {
	std::string("INITIAL"), std::string("CREATION_ERROR"),
	std::string("CREATED"), std::string("SEARCHING"), std::string("RECEIVED"),
	std::string("READY"), std::string("TIME_OUT"),
	std::string("PARSE_ERROR"), std::string("SOCK_ERROR"),
	std::string("CANCELLED"), std::string("TERMINATED"),
};

////////////////////////////////////////////////////////////////////////////
// UpResourceRecord methods
UpResourceRecord::UpResourceRecord(const ns_rr &rr, const ns_msg &msg) : next(0)
{
   switch(rr.type) {
      case ns_t_a:      rrPtr = new UpRRA(rr,msg);      return;
      case ns_t_aaaa:   rrPtr = new UpRRAAAA(rr,msg);   return;
      case ns_t_mx:     rrPtr = new UpRRMX(rr,msg);     return;
      case ns_t_soa:    rrPtr = new UpRRSOA(rr,msg);    return;
      case ns_t_naptr:  rrPtr = new UpRRNAPTR(rr,msg);  return;
      case ns_t_srv:    rrPtr = new UpRRSRV(rr,msg);    return;
      case ns_t_txt:    rrPtr = new UpRRSimple(rr,msg); return;
      case ns_t_cname:
      case ns_t_ptr:
      case ns_t_ns:     rrPtr = new UpRRDomain(rr,msg); return;
      default:          rrPtr = new UpRR(rr); return;
   }
}

std::ostream& operator<<(std::ostream& o, const UpResourceRecord& x) {
		switch(x.getType()) {
			case ns_t_a:		o << *((UpRRA*)x.rrPtr);		return o;
			case ns_t_aaaa:	o << *((UpRRAAAA*)x.rrPtr);	return o;
			case ns_t_mx:		o << *((UpRRMX*)x.rrPtr);		return o;
			case ns_t_soa:		o << *((UpRRSOA*)x.rrPtr);		return o;
			case ns_t_naptr:	o << *((UpRRNAPTR*)x.rrPtr);	return o;
			case ns_t_srv:		o << *((UpRRSRV*)x.rrPtr);		return o;
			case ns_t_txt:		o << *((UpRRSimple*)x.rrPtr);	return o;
			case ns_t_cname:
			case ns_t_ptr:
			case ns_t_ns:		o << *((UpRRDomain*)x.rrPtr);	return o;
			default:				o << *(x.rrPtr);					return o;
		}
	}

std::string& operator<<(std::string& s, const UpResourceRecord& x) {
		switch(x.getType()) {
			case ns_t_a:		s << *((UpRRA*)x.rrPtr);		return s;
			case ns_t_aaaa:	s << *((UpRRAAAA*)x.rrPtr);	return s;
			case ns_t_mx:		s << *((UpRRMX*)x.rrPtr);		return s;
			case ns_t_soa:		s << *((UpRRSOA*)x.rrPtr);		return s;
			case ns_t_naptr:	s << *((UpRRNAPTR*)x.rrPtr);	return s;
			case ns_t_srv:		s << *((UpRRSRV*)x.rrPtr);		return s;
			case ns_t_txt:		s << *((UpRRSimple*)x.rrPtr);	return s;
			case ns_t_cname:
			case ns_t_ptr:
			case ns_t_ns:		s << *((UpRRDomain*)x.rrPtr);	return s;
			default:				s << *(x.rrPtr);					return s;
		}
	}


////////////////////////////////////////////////////////////////////////////
// UpRR methods

int UpRR::domainExport(	std::string& dst,const unsigned char*& slider,
								const ns_msg& msg )
{
	dst.resize(MAXDNAME+1);
	int ret = ns_name_uncompress(	ns_msg_base(msg),ns_msg_end(msg),
											slider, (char*)dst.data(), MAXDNAME );
	if( ret>0 )	{ slider += ret; dst.resize(strlen(dst.data())); }
	else			{ dst.resize(0); }
	return ret;
}

int UpRR::stringExport(	std::string& dst, const unsigned char*& slider,
								const ns_msg& msg )
{
	unsigned int n((unsigned int)*slider++);
	const unsigned char* src=slider;
	slider+=n;
	if(slider>ns_msg_end(msg)) return -1;
	dst.assign((char*)src,n);
	return n;
}

////////////////////////////////////////////////////////////////////////////
UpRRMX::UpRRMX(const ns_rr &record, const ns_msg &msg)
: UpRR(record)
{
	const unsigned char *p=ns_rr_rdata(record),*q=p;
	shortExport(preference,p);
	if(domainExport(server,p,msg)<0) { type=ns_t_invalid; return; }
	if((p-q)>ns_rr_rdlen(record)) type=ns_t_invalid;
}

std::ostream& operator<<(std::ostream& o, const UpRRMX& urr) {
	o << urr.name <<"\tMX\t"<<urr.preference<<"\t"<< urr.server;
	return o;
}

std::string& operator<<(std::string& o, const UpRRMX& urr) {
	o << urr.name <<"\tMX\t"<<urr.preference<<"\t"<< urr.server;
	return o;
}

////////////////////////////////////////////////////////////////////////////
UpRRSOA::UpRRSOA(const ns_rr &record, const ns_msg &msg)
: UpRR(record)
{
	const unsigned char *p=ns_rr_rdata(record),*q=p;
	if(domainExport(origin,p,msg)<0)   { type=ns_t_invalid; return; }
	if(domainExport(mailAddr,p,msg)<0) { type=ns_t_invalid; return; }
	longExport(serial,p);
	longExport(refresh,p);
	longExport(retry,p);
	longExport(expire,p);
	longExport(minimum,p);
	if((p-q)>ns_rr_rdlen(record)) type=ns_t_invalid;
}

std::ostream& operator<<(std::ostream& o, const UpRRSOA& urr) {
	o	<< urr.name <<"\tSOA\t"<<urr.origin<<"\t"<< urr.mailAddr<<"\t"
		<<urr.serial<<"\t"<<urr.refresh<<"\t"<<urr.retry<<"\t"
		<<urr.expire <<"\t"<<urr.minimum;
	return o;
}

std::string& operator<<(std::string& o, const UpRRSOA& urr) {
	o	<< urr.name <<"\tSOA\t"<<urr.origin<<"\t"<< urr.mailAddr<<"\t"
		<<urr.serial<<"\t"<<urr.refresh<<"\t"<<urr.retry<<"\t"
		<<urr.expire <<"\t"<<urr.minimum;
	return o;
}

////////////////////////////////////////////////////////////////////////////
UpRRNAPTR::UpRRNAPTR(const ns_rr &record, const ns_msg &msg)
: UpRR(record)
{
	const unsigned char *p=ns_rr_rdata(record),*q=p;
	shortExport(order,p);
	shortExport(preference,p);
	if(stringExport(flags,p,msg)<0) { type=ns_t_invalid; return; }
	if(stringExport(service,p,msg)<0) { type=ns_t_invalid; return; }
	if(stringExport(regex,p,msg)<0) { type=ns_t_invalid; return; }
	copyToLower16((char*)flags.data(),flags.data());
	// rfc's: 3958,3761, and 3401,3402,3403,3404,3405 (last five obsolete 2915)
	// flags=="":  next step is a lookup for another NAPTR record
	// flags=="s": next step is a lookup for an SRV record
	// flags=="a": next step is a lookup for an A or AAAA record
	// flags=="u": output of the regex is an absoluteUri
	// flags=="p": protocol specific
	if( flags.empty() || flags[0]=='s'|| flags[0]=='a' ) {
		if(domainExport(replacement,p,msg)<0) { type=ns_t_invalid; return; }
	} else {
		if(stringExport(replacement,p,msg)<0) { type=ns_t_invalid; return; }
	}
	if((p-q)>ns_rr_rdlen(record)) type=ns_t_invalid;
}

std::ostream& operator<<(std::ostream& o, const UpRRNAPTR& urr) {
	o	<< urr.name <<"\tNAPTR\t"<<urr.order<<"\t"<< urr.preference<<"\t\""
		<<urr.flags<<"\"\t\""<<urr.service<<"\"\t\""<<urr.regex<<"\"\t";
	if( urr.flags.empty() || urr.flags[0]=='s'|| urr.flags[0]=='a' )
			o<<urr.replacement;
	else	o<<"\""<<urr.replacement<<"\"";
	return o;
}

std::string& operator<<(std::string& o, const UpRRNAPTR& urr) {
	o	<< urr.name <<"\tNAPTR\t"<<urr.order<<"\t"<< urr.preference<<"\t\""
		<<urr.flags<<"\"\t\""<<urr.service<<"\"\t\""<<urr.regex<<"\"\t";
	if( urr.flags.empty() || urr.flags[0]=='s'|| urr.flags[0]=='a' )
			o<<urr.replacement;
	else	o<<"\""<<urr.replacement<<"\"";
	return o;
}

////////////////////////////////////////////////////////////////////////////
UpRRSRV::UpRRSRV(const ns_rr &record, const ns_msg &msg)
: UpRR(record)
{
	const unsigned char *p=ns_rr_rdata(record),*q=p;
	shortExport(preference,p);
	shortExport(weight,p);
	shortExport(port,p);
	if(domainExport(target,p,msg)<0) { type=ns_t_invalid; return; }
	if((p-q)>ns_rr_rdlen(record)) type=ns_t_invalid;
}

std::ostream& operator<<(std::ostream& o, const UpRRSRV& urr) {
	o	<< urr.name <<"\tSRV\t"<<urr.preference<<"\t"<< urr.weight<<"\t"
		<<urr.port<<"\t"<<urr.target;
	return o;
}

std::string& operator<<(std::string& o, const UpRRSRV& urr) {
	o	<< urr.name <<"\tSRV\t"<<urr.preference<<"\t"<< urr.weight<<"\t"
		<<urr.port<<"\t"<<urr.target;
	return o;
}
/////////////////////////////////////////////////////////////////////////

DnsQueryType::DnsQueryType() {
	for(unsigned int t=0; t<256; t++)		qt[t]="";
	qm["INVALID"]	= ns_t_invalid;		qt[ns_t_invalid]="INVALID";
	qm["A"]			= ns_t_a;      		qt[ns_t_a]="A";
	qm["NS"]			= ns_t_ns;     		qt[ns_t_ns]="NS";
	qm["MD"]			= ns_t_md;     		qt[ns_t_md]="MD";
	qm["MF"]			= ns_t_mf;     		qt[ns_t_mf]="MF";
	qm["CNAME"]		= ns_t_cname;  		qt[ns_t_cname]="CNAME";
	qm["SOA"]		= ns_t_soa;    		qt[ns_t_soa]="SOA";
	qm["MB"]			= ns_t_mb;     		qt[ns_t_mb]="MB";
	qm["MG"]			= ns_t_mg;     		qt[ns_t_mg]="MG";
	qm["MR"]			= ns_t_mr;     		qt[ns_t_mr]="MR";
	qm["NULL"]		= ns_t_null;   		qt[ns_t_null]="NULL";
	qm["WKS"]		= ns_t_wks;    		qt[ns_t_wks]="WKS";
	qm["PTR"]		= ns_t_ptr;    		qt[ns_t_ptr]="PTR";
	qm["HINFO"]		= ns_t_hinfo;  		qt[ns_t_hinfo]="HINFO";
	qm["MINFO"]		= ns_t_minfo;  		qt[ns_t_minfo]="MINFO";
	qm["MX"]			= ns_t_mx;     		qt[ns_t_mx]="MX";
	qm["TXT"]		= ns_t_txt;    		qt[ns_t_txt]="TXT";
	qm["RP"]			= ns_t_rp;     		qt[ns_t_rp]="RP";
	qm["AFSDB"]		= ns_t_afsdb;  		qt[ns_t_afsdb]="AFSDB";
	qm["X25"]		= ns_t_x25;    		qt[ns_t_x25]="X25";
	qm["ISDN"]		= ns_t_isdn;   		qt[ns_t_isdn]="ISDN";
	qm["RT"]			= ns_t_rt;     		qt[ns_t_rt]="RT";
	qm["NSAP"]		= ns_t_nsap;   		qt[ns_t_nsap]="NSAP";
	qm["NSAP_PTR"]	= ns_t_nsap_ptr;	qt[ns_t_nsap_ptr]="NSAP_PTR";
	qm["SIG"]		= ns_t_sig;    		qt[ns_t_sig]="SIG";
	qm["KEY"]		= ns_t_key;    		qt[ns_t_key]="KEY";
	qm["PX"]			= ns_t_px;     		qt[ns_t_px]="PX";
	qm["GPOS"]		= ns_t_gpos;   		qt[ns_t_gpos]="GPOS";
	qm["AAAA"]		= ns_t_aaaa;   		qt[ns_t_aaaa]="AAAA";
	qm["LOC"]		= ns_t_loc;    		qt[ns_t_loc]="LOC";
	qm["NXT"]		= ns_t_nxt;    		qt[ns_t_nxt]="NXT";
	qm["EID"]		= ns_t_eid;    		qt[ns_t_eid]="EID";
	qm["NIMLOC"]	= ns_t_nimloc; 		qt[ns_t_nimloc]="NIMLOC";
	qm["SRV"]		= ns_t_srv;    		qt[ns_t_srv]="SRV";
	qm["ATMA"]		= ns_t_atma;   		qt[ns_t_atma]="ATMA";
	qm["NAPTR"]		= ns_t_naptr;  		qt[ns_t_naptr]="NAPTR";
	qm["TSIG"]		= ns_t_tsig;   		qt[ns_t_tsig]="TSIG";
	qm["IXFR"]		= ns_t_ixfr;   		qt[ns_t_ixfr]="IXFR";
	qm["AXFR"]		= ns_t_axfr;   		qt[ns_t_axfr]="AXFR";
	qm["MAILB"]		= ns_t_mailb;  		qt[ns_t_mailb]="MAILB";
	qm["MAILA"]		= ns_t_maila;  		qt[ns_t_maila]="MAILA";
	qm["ANY"]		= ns_t_any;    		qt[ns_t_any]="ANY";
}
