/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <cstring>
#include <cerrno>
#include <UpTools/UpLine.h>
#include <unistd.h>

#define initDefaultSepTable(t) \
{	bool* p = t; bool* q=defaultSeparator; int i=256; while(i--) *p++ = *q++; }

bool defaultSeparator[] = {
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 };


int UpLine::getBytes() {
	if( ( line == buf ) && ( next == endofbuf ) ) { 
		status = "longitud de linea mayor que el buffer";
		if( _fullBufIsLine ) return 0;
		return -1;
	} else {
		register size_t currentLineBytes = endofbytes - line; 
		if( line != buf ) {
			if( currentLineBytes>0 ) std::memmove(buf,line,currentLineBytes);
			line = buf;
			endofbytes = next = buf + currentLineBytes;
		}
		int r = readWTimer(endofbytes,BUFSIZE-currentLineBytes);
		if( r < 0 )  { return -1; }
		if( r == 0 ) { *next = '\0'; return 0; }
		endofbytes += r;
		return r;
	}
}

UpLine::UpLine(int fd)
: _fd(fd) , next(buf), endofbytes(buf), endofbuf(buf+BUFSIZE), status(""),
  _acceptUnterminatedLine(false), _fullBufIsLine(false), _joinSeparators(true),
  _crnl(true),lineLen(0),timerPtr(NULL),lastSeparator(0)
{
	*buf = '\0'; *(statbuf+STATSIZE) = '\0';
	initDefaultSepTable( (bool*) separator );
	FD_ZERO(&fdsetOriginal);
	FD_SET(_fd,&fdsetOriginal);
}

UpLine::UpLine()
: _fd(0) , next(buf), endofbytes(buf), endofbuf(buf+BUFSIZE), status(""),
  _acceptUnterminatedLine(false), _fullBufIsLine(false), _joinSeparators(true),
  _crnl(true),lineLen(0),timerPtr(NULL),lastSeparator(0)
{
	*buf = '\0'; *(statbuf+STATSIZE) = '\0';
	initDefaultSepTable( (bool*) separator );
	FD_ZERO(&fdsetOriginal);
	FD_SET(0,&fdsetOriginal);
	_fd1=_fd+1;
}


void UpLine::setFd(int fd)
{
	flush();
	_fd = fd;
	FD_ZERO(&fdsetOriginal);
	FD_SET(_fd,&fdsetOriginal);
	_fd1=_fd+1;
}

void UpLine::flush() {
	next = endofbytes = buf;
	status = "";
	lineLen = 0;
	lastSeparator = '\0';
}

void UpLine::setTimeout(time_t t) {
	timerPtr=&timer; timeout=t;
}

void UpLine::setTimeout(const UpTimeVal& upt) {
	timerPtr=&timer; timeout=upt;
}

void UpLine::setTimeout(const timeval& tv) {
	timerPtr=&timer; timeout=tv;
}

void UpLine::setNoTimeout() {
	timerPtr=NULL;
}

void UpLine::acceptUnterminatedLines(bool v)
{ _acceptUnterminatedLine=v; }

void  UpLine::acceptLongLines(bool v)
{ _fullBufIsLine=v; }

void  UpLine::acceptCRNL(bool v)
{ _crnl=v; }

void  UpLine::joinSeparators(bool v)
{ _joinSeparators=v; }

void  UpLine::setSeparators(const char* s, unsigned char separatorCount)
{
	std::memset(separator,0,256);
	while(separatorCount--) separator[(unsigned char)*s++] = true;
}

const char* UpLine::getStatus()
{ return status; }

size_t UpLine::getLen()
{ return lineLen; }

char UpLine::getLastSeparator()
{ return lastSeparator; }


const char* UpLine::getLine()
{
	if(timerPtr) timeout.copyTo(timer); // reset timer
	line = next;
	for(;;) {
		if( next == endofbytes) {
			int r = getBytes();
			if( r < 0 ) { lineLen = 0; return NULL; }
			if( r == 0 ) {
				if( _acceptUnterminatedLine ) {
					*next = '\0';
					lineLen = next-line;
					return lineLen ? line : NULL;
				}
				lineLen = 0;
				return NULL;
			} 
		}
		if( *next == '\n' ) {
			*next = '\0';
			lineLen = next-line;
			if(_crnl) {  // check if this a '\r\n'
				char* previous = next-1;
				if( (next>buf) && (*previous=='\r') ) {
					*previous='\0';
					lineLen--;
				}
			}
			next++;
			status = "ok";
			return line;
		}
		next++;
	}
}



const char* UpLine::getToken()
{
	if(timerPtr) timeout.copyTo(timer); // reset timer
	line = next;
	for(;;) {
		if( next == endofbytes) {
			int r = getBytes();
			if( r < 0 ) { lineLen = 0; return NULL; }
			if( r == 0 ) {
				if( _acceptUnterminatedLine ) {
					*next = '\0';
					lineLen = next-line;
					return lineLen ? line : NULL;
				}
				lineLen = 0;
				return NULL;
			} 
		}
		if( separator[(unsigned char)*next] ) {
			if( _joinSeparators && ( line == next ) ) {
				line++; // do not accept null tokens when joining separators
			} else {
				lastSeparator=*next;
				*next = '\0';
				lineLen = next-line;
				if(_crnl) {  // check if this a '\r\n'
					char* previous = next-1;
					if( (next>buf) && (*previous=='\r') ) {
						*previous='\0';
						lineLen--;
					}
				}
				next++;
				status = "ok";
				return line;
			}
		}
		next++;
	}
}



int UpLine::read(char* bytesBuf, size_t byteCount) {
	if(timerPtr) timeout.copyTo(timer); // reset timer
	size_t currentLineBytes = endofbytes - line;
	if( currentLineBytes > byteCount ) {
		std::memcpy( bytesBuf , line , byteCount );
		next += byteCount;
		status = "ok";
		return lineLen = byteCount;
	}
	if( currentLineBytes ) {
		std::memcpy( bytesBuf , line , currentLineBytes );
		next = endofbytes = buf; lineLen = 0;
		bytesBuf  += currentLineBytes;
		byteCount -= currentLineBytes;
	}
	if( byteCount ) {
		int r = ::read( _fd , bytesBuf , byteCount );
		if(r<0) {
			if( currentLineBytes ) {
				status = "ok";
				return lineLen = currentLineBytes;
			}
			status = strncpy(statbuf,::strerror(errno),STATSIZE);
			lineLen = 0;
			return -1;
		}
		status = "ok";
		return lineLen = currentLineBytes + r;
	}
	status = "ok"; 
	return lineLen = currentLineBytes;
}

int UpLine::readWTimer(char* where, int readCount) {
	int n;
	if(timerPtr) { // si ( timerPtr==&timer )
		memcpy( &rfdset, &fdsetOriginal, sizeof(fd_set) );
		memcpy( &efdset, &fdsetOriginal, sizeof(fd_set) );
		n = select(_fd1,&rfdset,NULL,&efdset,timerPtr);
		if( (n<0) || FD_ISSET(_fd,&efdset) ) { // error or exception
			status = ::strerror(errno);
			return -1;
		}
		if( n==0 ) { //timeout
			status = "timeout";
			return -1;
		}
		if( !FD_ISSET(_fd,&rfdset) ) {
			status = "unknown error in select";
			return -1;
		}
	}
	n = ::read( _fd , where ,readCount );
	if( n < 0 ) {
   	status = std::strncpy(statbuf,::strerror(errno),STATSIZE);
		return -1;
	}
	if( n == 0 ) status = "zero bytes read, may be eof or interrupted";
	return n;
}

/* From 'man 2 select':
       Three independent sets of descriptors are  watched.   Those  listed  in
       readfds will be watched to see if characters become available for read-
       ing (more precisely, to see if a read will not block - in particular, a
       file  descriptor  is also ready on end-of-file), those in writefds will
       be watched to see if a write will not block,  and  those  in  exceptfds
       will  be  watched  for  exceptions.   On exit, the sets are modified in
       place to indicate which descriptors actually changed status.
*/
