/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <UpTools/UpSmartLinkPtr.h>
#include <ostream>

/////////////////////////////////////////////////////////////////////////////
// UplinkableMonitor global object
UpLinkableMonitor upLinkableMonitor;

/////////////////////////////////////////////////////////////////////////////
// public methods

UpLinkableMonitor::UpLinkableMonitor()
: unnamedUpLinkables(0) { } 

UpLinkableMonitor::~UpLinkableMonitor()
{ }

// get the UpLinkable count
unsigned int UpLinkableMonitor::getCount()
{ return getUnnamedCount()+getNamedCount(); }

 
unsigned int UpLinkableMonitor::getUnnamedCount()
{ return unnamedUpLinkables; }


unsigned int UpLinkableMonitor::getNamedCount()
{ mx.lock();
  unsigned int n(0);
  CharPtrUIntMap::iterator it=namedUpLinkables.begin();
  while( it!=namedUpLinkables.end() ) {
    n += it->second;
    ++it;
  }
  mx.unlock();
  return n;
}


unsigned int UpLinkableMonitor::getCount(const char* classname)
{ mx.lock();
  CharPtrUIntMap::iterator it=namedUpLinkables.find(classname);	
  unsigned int n( (it!=namedUpLinkables.end()) ? it->second : 0 );
  mx.unlock();
  return n;
}


// reports of UpLinkable counts to ostream
void UpLinkableMonitor::report(std::ostream& o)
{ reportUnnamed(o);
  reportNamed(o);
}


void UpLinkableMonitor::reportUnnamed(std::ostream& o)
{ o<<"Unnamed UpLinkable count: "<<unnamedUpLinkables<<'\n'; }


void UpLinkableMonitor::reportNamed(std::ostream& o)
{ mx.lock();
  CharPtrUIntMap::iterator it=namedUpLinkables.begin();
  while( it!=namedUpLinkables.end() ) {
    o<<it->first<<" count: "<<it->second<<'\n';
    ++it;
  }
  mx.unlock();
}


void UpLinkableMonitor::report(std::ostream& o,const char* classname)
{ o<<classname<<" count: "<<getCount(classname)<<'\n'; }


/////////////////////////////////////////////////////////////////////////////
// private methods

_Atomic_word UpLinkableMonitor::constructorEvent()
{  return __gnu_cxx::__exchange_and_add(&unnamedUpLinkables,1)+1; }


_Atomic_word UpLinkableMonitor::constructorEvent(const char* _className) {
	_Atomic_word upl;
	if(_className) {
		mx.lock();
			upl=++(namedUpLinkables[_className]);
		mx.unlock();
	} else {
		upl=__gnu_cxx::__exchange_and_add(&unnamedUpLinkables,1)+1;
	}
	return upl;
}


_Atomic_word UpLinkableMonitor::destructorEvent(const char* _className) {
	_Atomic_word upl;
	if(_className) {
		mx.lock();
			upl=--(namedUpLinkables[_className]);
		mx.unlock();
	} else {
		upl=__gnu_cxx::__exchange_and_add(&unnamedUpLinkables,-1)-1;
	}
	return upl;
}

