/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <UpTools/UpLog.h>



extern "C" {


/******************************************************************************/
/******************************************************************************/
/* internal functions and data */

int upLogPid     = 0;
int upLogThid    = 0;
int upLogPerror  = 0;
int upLogUpTo    = LOG_DEBUG;
int upLogFd      = 2;
const char* upLogIdent = "";
char upLogIdentBuf[64];



int upLogHdrNoThreads(char* buf) {
	char *p=buf+24;
	const char *q=upLogIdent;
	time_t t; time(&t); ctime_r(&t,buf); /* 24 bytes */
	*p++=' '; while( *q ) *p++=*q++; 
	return upLogPid? (p-buf+sprintf(p,"[%u] ",(unsigned int)getpid()))
						: (p-buf);
}

int (*upLogHdr)(char*buf) = upLogHdrNoThreads;



void upLogInternalSetOptions(const char *ident, int options,int logLevel)
{
	upLogPid  	 = (options&LOG_PID)?1:0;
	upLogThid  	 = (options&LOG_THID)?1:0;
	upLogPerror  = (options&LOG_PERROR)?1:0;
	upLogUpTo    = logLevel;
	upLogFd      = -1;
	if( ident && *ident ) {
		strncpy( upLogIdentBuf,ident,63 );
		upLogIdentBuf[63]=0;
		upLogIdent = upLogIdentBuf;
	}
}



/* set UPLOG to use syslogd */
void upOpenLogInternal(const char *ident, int options, int facility,
							  int logLevel)
{
	upLogInternalSetOptions(ident,options,logLevel);
	openlog(ident,options,facility);
	if( upLogThid ) UPLOG(LOG_WARNING,"upOpenLog(...) warning: to log thread id "
								 "(LOG_THID) replace upOpenLog with upOpenLogFile");
}


/* set UPLOG to use file */
void upOpenLogFileInternal(const char* ident, int options,
									const char* filename,int logLevel,
									int (*upLogHdrFunc)(char*) )
{
	upLogInternalSetOptions(ident,options,logLevel);
	if( strcasecmp("stdout",filename)==0 ) {
		upLogFd=1;
	} else if( strcasecmp("stderr",filename)==0 ) {
		upLogFd=2; upLogPerror=0; /* no need to log twice the same thing */
	} else {
		upLogFd=open(filename,O_WRONLY|O_CREAT|O_APPEND|O_NONBLOCK,00644);
		if(upLogFd<0) {
			upLogFd=2; upLogPerror=0;
			UPLOG(LOG_ERR,"upOpenLogFile() error opening file %s"
							  ", logging to stderr",filename);
		}
	}
	if( upLogThid ) {
		if( upLogHdrFunc!=0 ) upLogHdr=upLogHdrFunc;
		else {
			const char* m1=
				"upOpenLogFile(...) warning: LOG_THID set but UpThread.h not included.";
			const char* m2=
				"To use LOG_THID include UpTools/UpThread.h in the file calling "
				"upOpenLogFile(...) and link using -lpthread";
			UPLOG(LOG_WARNING,m1); UPLOG(LOG_WARNING,m2);
		}
	}
}

/* close log file */
void upCloseLog() {
	if(upLogFd>2) close(upLogFd);
	if(upLogFd<0) closelog();
}




} // end of extern "C"
