/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <UpTools/UpOp.h>
#include <cmath>



/////////////////////////////////////////////////////////////////////
#if __WORDSIZE==64 || defined(__s390__)
std::string& operator<<(std::string& s,size_t n) {
	char buf[32],*p=buf+31; *p=0;
   do { *(--p) = '0'+(n%10); } while(n/=10);
	s.append(p,31-(p-buf));
   return s;
}
#endif

std::string& operator<<(std::string& s,unsigned int n) {
	char buf[32],*p=buf+31; *p=0;
   do { *(--p) = '0'+(n%10); } while(n/=10);
	s.append(p,31-(p-buf));
   return s;
}

std::string& operator<<(std::string& s,unsigned long long n) {
	char buf[32],*p=buf+31; *p=0;
   do { *(--p) = '0'+(n%10); } while(n/=10);
	s.append(p,31-(p-buf));
   return s;
}


std::string& operator<<(std::string& s,int n) {
	char buf[32],*p=buf+31; *p=0;
	if(n<0) {
		n = -n;
		do { *(--p) = '0'+(n%10); } while(n/=10);
		*(--p)='-';
	} else {
		do { *(--p) = '0'+(n%10); } while(n/=10);
	}
	s.append(p,31-(p-buf));
   return s;
}



std::string& operator<<(std::string& s,long long n) {
	char buf[32],*p=buf+31; *p=0;
	if(n<0) {
		n = -n;
		do { *(--p) = '0'+(n%10); } while(n/=10);
		*(--p)='-';
	} else {
		do { *(--p) = '0'+(n%10); } while(n/=10);
	}
	s.append(p,31-(p-buf));
   return s;
}


//////////////////////////////////////////////////////////////////////

unsigned int& operator<<(unsigned int& n,const std::string& s) {
	size_t len = s.size();
   if(len==0) return (n=0);
	const char* p = s.c_str();

	if( !std::isdigit(*p) ) return (n=0);
  	n = (unsigned int) ((*p++)-'0');

   while( (--len) && std::isdigit(*p) ) {
		 n *= 10;
		 n += (unsigned int) ((*p++)-'0');
	}
   return n;
}

unsigned long long& operator<<(unsigned long long& n,const std::string& s) {
	size_t len = s.size();
   if(len==0) return (n=0);
	const char* p = s.c_str();

	if( !std::isdigit(*p) ) return (n=0);
  	n = (unsigned long long) ((*p++)-'0');

   while( (--len) && std::isdigit(*p) ) {
		 n *= 10;
		 n += (unsigned long long) ((*p++)-'0');
	}
   return n;
}

#if __WORDSIZE==64 || defined(__s390__)
size_t& operator<<(size_t& n,const std::string& s) {
	size_t len = s.size();
   if(len==0) return (n=0);
	const char* p = s.c_str();

	if( !std::isdigit(*p) ) return (n=0);
  	n = (size_t) ((*p++)-'0');

   while( (--len) && std::isdigit(*p) ) {
		 n *= 10;
		 n += (size_t) ((*p++)-'0');
	}
   return n;
}
#endif

int& operator<<(int& n,const std::string& s) {
	size_t len = s.size();
   if(len==0) return (n=0);
	const char* p = s.c_str();
	bool neg=false; if(*p=='-') { p++; neg=true; }
	if( !std::isdigit(*p) ) return (n=0);
  	n = (int) ((*p++)-'0');

   while( (--len) && std::isdigit(*p) ) {
		 n *= 10;
		 n += (int) ((*p++)-'0');
	}
	if(neg) n = -n;
   return n;
}


long long& operator<<(long long& n,const std::string& s) {
	size_t len = s.size();
   if(len==0) return (n=0);
	const char* p = s.c_str();
	bool neg=false; if(*p=='-') { p++; neg=true; }
	if( !std::isdigit(*p) ) return (n=0);
  	n = (long long) ((*p++)-'0');

   while( (--len) && std::isdigit(*p) ) {
		 n *= 10;
		 n += (long long) ((*p++)-'0');
	}
	if(neg) n = -n;
   return n;
}


#define _EXPO_PROC \
      if(*p=='e' || *p=='E' ) { \
			if(minus) d=-d; \
         if(*(++p)==0) return d; \
         double q=10; \
         if(*p=='-') { \
            if(*(++p)==0) return d; \
            q=0.1; \
         } \
         unsigned int e= (unsigned int) std::atoi(p) ; \
         if(e>300) { \
				d = ( q==10 ? (minus? -INFINITY:INFINITY) : 0 ) ; \
				return d; \
			} \
         while(e--) d *=q; \
         return d; \
      }


double& operator<<(double& d, const std::string& s ) {
   d = 0;
	const char* p=s.c_str();
   if(!*p) return d;
   bool minus=false;
   if(*p=='-') { minus=true; p++; }
   while(*p) {
      _EXPO_PROC;
      if(*p=='.') {
         double frac=0.1; p++;
         while(*p) {
            _EXPO_PROC;
            d += frac * ((double)((*p++)-'0'));
            frac *= 0.1;
         }
         break;
      }
		if( ! std::isdigit(*p) ) break;
		d *= 10;
      d += (double) ((*p++)-'0');
   }
	if(minus) d=-d;
	return d;
}

#define _EXPO_PROCF \
      if(*p=='e' || *p=='E' ) { \
			if(minus) d=-d; \
         if(*(++p)==0) return d; \
         float q=10; \
         if(*p=='-') { \
            if(*(++p)==0) return d; \
            q=0.1; \
         } \
         unsigned int e= (unsigned int) std::atoi(p) ; \
         if(e>300) { \
				d = ( q==10 ? (minus? -INFINITY:INFINITY) : 0 ) ; \
				return d; \
			} \
         while(e--) d *=q; \
         return d; \
      }

float& operator<<(float& d, const std::string& s ) {
   d = 0;
	const char* p=s.c_str();
   if(!*p) return d;
   bool minus=false;
   if(*p=='-') { minus=true; p++; }
   while(*p) {
      _EXPO_PROCF;
      if(*p=='.') {
         float frac=0.1; p++;
         while(*p) {
            _EXPO_PROC;
            d += frac * ((float)((*p++)-'0'));
            frac *= 0.1;
         }
         break;
      }
		if( ! std::isdigit(*p) ) break;
		d *= 10;
      d += (float) ((*p++)-'0');
   }
	if(minus) d=-d;
	return d;
}



bool& operator<<(bool& b, const std::string& s ) {
	if( s.empty() || s=="0" || s=="false" || s=="FALSE" || s=="no" || s=="NO" )
		b=false;
	else
		b=true;
	return b;
}

std::string& operator<<(std::string& s, bool b) {
	if( b ) s.append("true",4);
	else    s.append("false",5);
	return s;
}








// strips a string from leading and trailing spaces, tabs and control chars
void chomp(std::string& s) { // assumes us-ascii and signed char type
 	register const char* p=s.data();
   register const char* q=p+s.size();
   while( ( p<q ) && ( *p<=' ' ) ) p++;
   while( ( p<q ) && ( *q<=' ' ) ) q--;
   if( p<q ) s = s.substr( (p-s.data()) , (q-p+1) );
   else s.clear();
}

// strips a string from spaces, tabs and control chars
void join(std::string& s) { // assumes us-ascii and signed char type
   register const char* p(s.c_str());
   register char* q((char*)p);
   while( *p != 0 ) {
      if( *p > ' ' ) {
         if( p != q ) *q = *p;
         ++q;
      }
      ++p;
   }
   s.resize(q-s.c_str());
}

