/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <UpTools/UpSqlConnPool.h>
#include <UpTools/UpRegex.h>


UpSqlConnPool::UpSqlConnPool()
: firstConn(0), connCount(0), basePtr(0)
{ }

UpSqlConnPool::UpSqlConnPool(const UpSqlConn& modelConn)
: firstConn(0), connCount(0), basePtr(modelConn.newCopyConn())
{ }


void UpSqlConnPool::setConnectionModel(const UpSqlConn& modelConn) {
	if(basePtr) delete basePtr;
	basePtr = modelConn.newCopyConn();
}

UpSqlConnPool::~UpSqlConnPool() {
	if( basePtr ) delete basePtr;
	AuxThreadConn *next;
	while(firstConn)
		{ next=firstConn->next; delete firstConn; firstConn=next; }
}


UpSqlConn& UpSqlConnPool::getConn() {
	UpSqlConn* connPtr;
	pthread_t self = pthread_self();
	mx.lock();
	if( firstConn==0 ) {
		if( basePtr ) { 
			firstConn = new AuxThreadConn(self,*basePtr);
			++connCount;
			connPtr = firstConn->connPtr;
		} else {
			connPtr = &upSqlConnUndef;
		}
	} else {
		AuxThreadConn* ptr;
		for( ptr=firstConn; ptr->self!=self; ptr=ptr->next )
			if( ptr->next==0 ) {
				ptr = ptr->next = new AuxThreadConn(self,*basePtr);
				++connCount;
				break;
			}
			connPtr = ptr->connPtr;
	}
	mx.unlock();
	return *connPtr;
}	



bool UpSqlConnPool::addHost(const std::string& host,bool pushFront) {
	bool retVal(false);
	mx.lock();
		if( basePtr ) {
			basePtr->addHost(host,pushFront);
			retVal=true;
			distribParamsUnsafe();
	}
	mx.unlock();
	return retVal;
}	

bool UpSqlConnPool::set( const std::string& sqlUri ) {
	bool retVal(false);
	mx.lock();
		if( basePtr and basePtr->set(sqlUri) ) {
			retVal = true;
			distribParamsUnsafe();
		}
	mx.unlock();
	return retVal;
}

bool UpSqlConnPool::set( 
	const std::string& _host, const std::string& _user,
	const std::string& _pass, const std::string& _dbname,	unsigned short _port,
	const std::string& _params )
{
	bool retVal(false);
	mx.lock();
		if(basePtr and basePtr->set(_host,_user,_pass,_dbname,_port,_params) ) {
			retVal = true;
			distribParamsUnsafe();
		}
	mx.unlock();
	return retVal;
}
 

/* In some systems pthread_t may be a structure instead of an int.
 * In C langages structures cannot be compared and
 * ptr->self!=self should be replaced by pthread_equal(ptr->self,self)==0
 */

void UpSqlConnPool::distribParamsUnsafe() {
	for( AuxThreadConn* ptr=firstConn; ptr!=0; ptr=ptr->next )
		ptr->connPtr->copyParams(*basePtr);
}
