/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <UpTools/UpSqlPgsql.h>
#include <UpTools/UpLog.h>
#include <UpTools/UpArch.h>


void UpSqlConnPgsql::disconnect() {
   if(need_close) {
      PQfinish(conn);
      need_close = false;
      statusOk = false;
      statusPhrase = NOT_CONNECTED;
   }
}


bool UpSqlConnPgsql::privateConnect(const std::string& host)
{
	const char* portStr=0;
	if( port>0 ) {
		char portBuf[16];
		std::snprintf(portBuf,15,"%u",(unsigned int)port);
		portStr = portBuf;
	}
	scrPass();
	conn = PQsetdbLogin(	host.c_str(), portStr, connParams.c_str(), 0 /*tty*/,
							   dbname.c_str(), user.c_str(), pass.c_str() );
	scrPass();

	if( conn==0 ) {
		need_close = false;
		statusOk = false;
		statusPhrase = PQerrorMessage(conn);
		UPLOG(LOG_ERR,"UpSqlConnPgsql::connect(host) failed connection to %s: %s",
            host.c_str(),statusPhrase);
		return false;
	}
	need_close = true;
	statusOk = true;
	statusPhrase = "ok";
	return statusOk;
}




bool UpSqlConnPgsql::exec(UpSqlQuery& q,const char* sqlText) { // null terminated query
	if( !statusOk and !connect() )
		{ q.setError(CONNECTION_BAD,statusPhrase); return false; }

	if( PQsendQuery(conn,sqlText) != 1 ) {
		// query fails for the first time (it does not necessary means a
		// connection error)
		int          queryErr    = PQstatus(conn);
		const char*  queryErrStr = PQerrorMessage(conn); 
		if( !errorIsConnectionError(queryErr) ) {
			// this time it is a query error, not a connection error
			q.setError(queryErr,queryErrStr);
			return false;
		}
		// this time it is a connection error, try to reconnect
		UPLOG(LOG_ERR,"UpSqlConnPgsql::exec() %s, trying to reconnect",queryErrStr);
		if( !connect() ) {
			// reconnection failed, sc status has been set in connect();
			q.setError(queryErr,queryErrStr);
         return false;
      }

		if( PQsendQuery(conn,sqlText) != 1 ) {
			int          queryErr    = PQstatus(conn);
			const char*  queryErrStr = PQerrorMessage(conn);
			// update query status
			q.setError(queryErr,queryErrStr);
			// we only update connection status if it is a connection error
			if( errorIsConnectionError(queryErr) ) {
				statusPhrase = queryErrStr;
				statusOk = false;
			}
			return false;
		}
		// go on and process query results
	}
	// query succeded
	statusPhrase = "ok";
	statusOk = true;
	q.clearResultList();
	PGresult *res;
	while( 0!=(res=PQgetResult(conn)) )	{
		q.addQueryElem(
			new QueryElemPgsql( res ) // agregar PQoidValue(_res) cuando corresponda para el lastId
		);
   }
   return true;
}


bool UpSqlConnPgsql::errorIsConnectionError(unsigned int err) {
	return err!=CONNECTION_OK;
}
