/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <UpTools/UpTimeVal.h>

// applications may modify this
// (see man strftime)
const char* UPTVdateFormat="%d-%b-%Y";
const char* UPTVtimeFormat="%H:%M:%S";
const char* UPTVdateTimeFormat="%d-%b-%Y %H:%M:%S";



bool UpTimeVal::readDate(const char* d) {
	struct tm _tm={0,0,0,0,0,0,0,0,0};
	if( 0==strptime(d,UPTVdateFormat,&_tm) ) return false;
	// tv_sec  = (_tm.tm_yday+(_tm.tm_year-70)*365+((_tm.tm_year-69)>>2))*86400;
	tv_sec  = mktime(&_tm);
	tv_usec = 0;
	return true;
}

bool UpTimeVal::readDateTime(const char* dt) {
	struct tm _tm={0,0,0,0,0,0,0,0,0};
	if( 0==strptime(dt,UPTVdateTimeFormat,&_tm) ) return false;
	tv_sec  = mktime(&_tm);
	tv_usec = 0;
	return true;
}


size_t UpTimeVal::charDate(char* buf,size_t bufsize) const {
   if(!buf) return 0;
	struct tm localTm;
	localtime_r(&tv_sec,&localTm);
	return strftime(buf,bufsize,UPTVdateFormat,&localTm);
}


size_t UpTimeVal::charTime(char* buf,size_t bufsize) const {
   if(!buf) return 0;
	struct tm localTm;
	localtime_r(&tv_sec,&localTm);
	return strftime(buf,bufsize,UPTVtimeFormat,&localTm);
}

size_t UpTimeVal::charDateTime(char* buf,size_t bufsize) const {
  if(!buf) return 0;
  struct tm localTm;
  localtime_r(&tv_sec,&localTm);
  return strftime(buf,bufsize,UPTVdateTimeFormat,&localTm);
}


void UpTimeVal::basicCout(std::ostream& o,const UpTimeVal& t) {
   o << t.tv_sec; if( t.tv_usec==0 ) return;
	o << '.';
   if     (t.tv_usec<10) o<<"00000";
   else if(t.tv_usec<100) o<<"0000";
   else if(t.tv_usec<1000) o<<"000";
   else if(t.tv_usec<10000) o<<"00";
   else if(t.tv_usec<100000) o<<'0';
   o<<t.tv_usec;
}

void UpTimeVal::basicSout(std::string& s,const UpTimeVal& t) {
   s << t.tv_sec; if( t.tv_usec==0 ) return;
	s.append(1,'.');
   if     (t.tv_usec<10) s.append("00000",5);
   else if(t.tv_usec<100) s.append("0000",4);
   else if(t.tv_usec<1000) s.append("000",3);
   else if(t.tv_usec<10000) s.append("00",2);
   else if(t.tv_usec<100000) s.append(1,'0');
   s<<t.tv_usec;
}

