/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPDNSBASE_INL
#define UPDNSBASE_INL

// This file contains inline code for UpDnsQueries and UpResolvers
// and also internal class definitions


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// RR classes: this are classes that need not to be known by the user.
//  They are used internally from the UpResourceRecord.

// SOA:   getOrigin(), getMailaddr() , getSerial(),
//        getRefresh(), getRetry(), getExpire(), getMinimum()
// NAPTR: getOrder(), getPreference(), getFlags(), getService(), getRegex(),
//        getReplacement()
// SRV:   getPreference(), getWeight(), getPort(), getTarget()
// MX:    getPreference(), getServer()
// A:     getAddr()
// AAAA:  getAddr()
// PTR:   getString()
// CNAME: getString()
// NS:    getString()
// TXT:   getString()


		
class UpRR {
public:
	UpRR(const ns_rr& rr);
	virtual ~UpRR();
	ns_type 								getType() const;		// all
	const std::string& 				getName() const;		// all
	virtual const std::string&		getString() const;	// all
	virtual const UpAddress&		getAddr() const;		// A,AAAA
	virtual const std::string& 	getServer() const;	// NS,MX
	virtual const std::string&		getOrigin() const;	// SOA
	virtual const std::string&		getMailAddr() const;	// SOA
	virtual unsigned int				getSerial() const;	// SOA
	virtual unsigned int				getRefresh() const;	// SOA
	virtual unsigned int				getRetry() const;		// SOA
	virtual unsigned int				getExpire() const;	// SOA
	virtual unsigned int				getMinimum() const;	// SOA
	virtual unsigned short			getOrder() const;		// NAPTR
	virtual unsigned short			getPreference() const;	// NAPTR,MX,SRV
	virtual const std::string&		getFlags() const;		// NAPTR
	virtual const std::string&		getService() const;	// NAPTR
	virtual const std::string&		getRegex() const;		// NAPTR
	virtual const std::string&		getReplacement() const;	// NAPTR
	virtual unsigned short			getWeight() const;	// SRV
	virtual unsigned short			getPort() const;		// SRV
	virtual const std::string&		getTarget() const;	// SRV
	friend std::ostream& operator<<(std::ostream& o, const UpRR& urr);
	friend std::string& operator<<(std::string& o, const UpRR& urr);
protected:
	ns_type		type;
	std::string	name;
	static void	shortExport(unsigned short& dst, const unsigned char*& slider);
	static void longExport(unsigned int& dst, const unsigned char*& slider);
	static int  stringExport(std::string& dst, const unsigned char*& slider,
									 const ns_msg& msg );
	static int  domainExport(std::string& dst, const unsigned char*& slider,
									 const ns_msg& msg );
private:
	UpRR(bool); // private constructor for invalid RR
	friend class UpResourceRecord;
	static std::string notApplicable;
	static UpAddress emptyAddr;
};

/////////////////////////////////////////////////////////////////////////////
class UpRRDomain : public UpRR {
public:
	UpRRDomain(const ns_rr &rr, const ns_msg &msg);
	const std::string& getString() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRDomain& urr);
	friend std::string& operator<<(std::string& o, const UpRRDomain& urr);
private:
	std::string dname;
};
/////////////////////////////////////////////////////////////////////////////
class UpRRSimple : public UpRR {
public:
	UpRRSimple(const ns_rr &rr, const ns_msg &msg);
	const std::string& getString() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRSimple& urr);
	friend std::string& operator<<(std::string& o, const UpRRSimple& urr);
private:
	std::string txt;
};
/////////////////////////////////////////////////////////////////////////////
class UpRRA : public UpRR {
public:
	UpRRA(const ns_rr &rr, const ns_msg &msg);
	const UpAddress& getAddr() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRA& urr);
	friend std::string& operator<<(std::string& o, const UpRRA& urr);
private:
	UpAddress upa;
};
/////////////////////////////////////////////////////////////////////////////
class UpRRAAAA : public UpRR {
public:
	UpRRAAAA(const ns_rr &rr, const ns_msg &msg);
	const UpAddress& getAddr() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRAAAA& urr);
	friend std::string& operator<<(std::string& o, const UpRRAAAA& urr);
private:
	UpAddress upa;
};
/////////////////////////////////////////////////////////////////////////////
class UpRRMX: public UpRR {
public:
	UpRRMX(const ns_rr &rr, const ns_msg &msg);
	const std::string& getServer() const;
	unsigned short getPreference() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRMX& urr);
	friend std::string& operator<<(std::string& o, const UpRRMX& urr);
private:
	std::string server;	
	unsigned short preference;
};
	
/////////////////////////////////////////////////////////////////////////////
class UpRRSOA: public UpRR {
public:
	UpRRSOA(const ns_rr &rr, const ns_msg &msg);
	const std::string&		getOrigin() const;
	const std::string&		getMailAddr() const;
	unsigned int				getSerial() const;
	unsigned int				getRefresh() const;
	unsigned int				getRetry() const;
	unsigned int				getExpire() const;
	unsigned int				getMinimum() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRSOA& urr);
	friend std::string& operator<<(std::string& o, const UpRRSOA& urr);
private:
	std::string origin,mailAddr;
	unsigned int serial,refresh,retry,expire, minimum;
};
	
/////////////////////////////////////////////////////////////////////////////
class UpRRNAPTR: public UpRR {
public:
	UpRRNAPTR(const ns_rr &rr, const ns_msg &msg);
	unsigned short				getOrder() const;
	unsigned short				getPreference() const;
	const std::string&		getFlags() const;
	const std::string&		getService() const;
	const std::string&		getRegex() const;
	const std::string&		getReplacement() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRNAPTR& urr);
	friend std::string& operator<<(std::string& o, const UpRRNAPTR& urr);
private:
	std::string flags,service,regex,replacement;
	unsigned short order,preference;
};
/////////////////////////////////////////////////////////////////////////////
class UpRRSRV: public UpRR {
public:
	UpRRSRV(const ns_rr &rr, const ns_msg &msg);
	const std::string&		getTarget() const;
	unsigned short				getPreference() const;
	unsigned short				getWeight() const;
	unsigned short				getPort() const;
	friend std::ostream& operator<<(std::ostream& o, const UpRRSRV& urr);
	friend std::string& operator<<(std::string& o, const UpRRSRV& urr);
private:
	std::string target;
	unsigned short preference,weight,port;
};
/////////////////////////////////////////////////////////////////////////////
/// end of RR classes declarations //////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////
// inline methods
///////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// UpResourceRecord inline methods
inline
UpResourceRecord::UpResourceRecord(bool q)
	: rrPtr(new UpRR(q)), next(0) { }
inline
UpResourceRecord::~UpResourceRecord()
	{ delete rrPtr; }
inline
unsigned int			UpResourceRecord::getSerial() const
	{ return rrPtr->getSerial(); }
inline
unsigned int			UpResourceRecord::getRefresh() const
	{ return rrPtr->getRefresh(); }
inline
unsigned int			UpResourceRecord::getMinimum() const
	{ return rrPtr->getMinimum(); }
inline
unsigned int			UpResourceRecord::getRetry() const
	{ return rrPtr->getRetry(); }
inline
unsigned int			UpResourceRecord::getExpire() const
	{ return rrPtr->getExpire();}
inline
const std::string&	UpResourceRecord::getMailAddr() const
	{ return rrPtr->getMailAddr(); }
inline
unsigned short			UpResourceRecord::getPreference() const
	{ return rrPtr->getPreference(); }
inline
const std::string&	UpResourceRecord::getService() const
	{ return rrPtr->getService(); }
inline
const std::string& 	UpResourceRecord::getReplacement() const
	{ return rrPtr->getReplacement(); }
inline
const std::string& 	UpResourceRecord::getServer() const
	{ return rrPtr->getServer(); }
inline
ns_type					UpResourceRecord::getType() const
	{ return rrPtr->getType(); }
inline
const std::string& 	UpResourceRecord::getName() const
	{ return rrPtr->getName(); }
inline
const std::string& 	UpResourceRecord::getString() const
	{ return rrPtr->getString(); }
inline
const UpAddress&		UpResourceRecord::getAddr() const
	{ return rrPtr->getAddr(); }
inline
const std::string&	UpResourceRecord::getOrigin() const
	{ return rrPtr->getOrigin(); }
inline
unsigned short			UpResourceRecord::getWeight() const
	{ return rrPtr->getWeight();}
inline
unsigned short			UpResourceRecord::getPort() const
	{ return rrPtr->getPort(); }
inline
unsigned short			UpResourceRecord::getOrder() const
	{ return rrPtr->getOrder(); }
inline
const std::string&	UpResourceRecord::getFlags() const
	{ return rrPtr->getFlags(); }
inline
const std::string&	UpResourceRecord::getRegex() const
	{ return rrPtr->getRegex(); }
inline
const std::string&  UpResourceRecord::getTarget() const
	{ return rrPtr->getTarget(); }

/////////////////////////////////////////////////////////////////////////////
// UpRR inline methods
inline
UpRR::UpRR(const ns_rr& rr) : type(ns_rr_type(rr)), name(ns_rr_name(rr)) { }
inline
UpRR::UpRR(bool q) : type(ns_t_invalid), name("invalid") { }
inline
UpRR::~UpRR() {}
inline
ns_type 					UpRR::getType() const     { return type; }
inline
const std::string&	UpRR::getName() const     { return name; }
inline
const std::string& 	UpRR::getString() const   { return notApplicable; }
inline
const std::string& 	UpRR::getServer() const   { return notApplicable; }
inline
const UpAddress&		UpRR::getAddr() const     { return emptyAddr; }
inline
unsigned int			UpRR::getSerial() const   { return 0; }
inline
unsigned int			UpRR::getRefresh() const  { return 0; }
inline
unsigned int			UpRR::getRetry() const    { return 0; }
inline
unsigned int			UpRR::getExpire() const   { return 0; }
inline
unsigned int			UpRR::getMinimum() const  { return 0; }
inline
const std::string&	UpRR::getOrigin() const   { return notApplicable; }
inline
const std::string&	UpRR::getMailAddr() const { return notApplicable; }
inline
unsigned short			UpRR::getWeight() const   { return 0; }
inline
unsigned short			UpRR::getPort() const     { return 0; }
inline
unsigned short			UpRR::getOrder() const    { return 0; }
inline
unsigned short			UpRR::getPreference() const { return 0; }
inline
const std::string&	UpRR::getFlags() const    { return notApplicable; }
inline
const std::string&	UpRR::getService() const  { return notApplicable; }
inline
const std::string&	UpRR::getRegex() const    { return notApplicable; }
inline
const std::string&	UpRR::getReplacement() const { return notApplicable; }
inline
const std::string&	UpRR::getTarget() const   { return notApplicable; }
inline
void UpRR::shortExport(unsigned short& dst, const unsigned char*& slider)
 { dst=ns_get16(slider); slider+=NS_INT16SZ; }
inline
void UpRR::longExport(unsigned int& dst, const unsigned char*& slider)
 { dst=ns_get32(slider); slider+=NS_INT32SZ; }
inline
std::ostream& operator<<(std::ostream& o, const UpRR& urr)
	{ o << urr.name << "\t" << dnsQueryType(urr.type) ; return o; }
inline
std::string& operator<<(std::string& o, const UpRR& urr)
	{ o << urr.name << "\t" << dnsQueryType(urr.type) ; return o; }
////////////////////////////////////////////////////////////////////////////
// inline methods for UpRRDomain
inline
UpRRDomain::UpRRDomain(const ns_rr &record, const ns_msg &msg)
	: UpRR(record)
	{ const unsigned char *p=ns_rr_rdata(record); domainExport(dname,p,msg); }
inline
const std::string& UpRRDomain::getString() const
	{ return dname; }
inline
std::ostream& operator<<(std::ostream& o, const UpRRDomain& urr)
	{ o << urr.name << "\t"<<dnsQueryType(urr.type)<<"\t" << urr.dname; return o; }
inline
std::string& operator<<(std::string& o, const UpRRDomain& urr)
	{ o << urr.name << "\t"<<dnsQueryType(urr.type)<<"\t" << urr.dname; return o; }

////////////////////////////////////////////////////////////////////////////
// inline methods for UpRRSimple
inline
UpRRSimple::UpRRSimple(const ns_rr &record, const ns_msg &msg)
	: UpRR(record)
	{ const unsigned char *p=ns_rr_rdata(record); stringExport(txt,p,msg); }
inline
const std::string& UpRRSimple::getString() const
	{ return txt; }
inline
std::ostream& operator<<(std::ostream& o, const UpRRSimple& urr)
	{ o << urr.name << "\t"<<dnsQueryType(urr.type)<<"\t\"" << urr.txt<<"\""; return o; }
inline
std::string& operator<<(std::string& o, const UpRRSimple& urr)
	{ o << urr.name << "\t"<<dnsQueryType(urr.type)<<"\t\"" << urr.txt<<"\""; return o; }

/////////////////////////////////////////////////////////////////////////////
// UpRRA inline methods
inline
UpRRA::UpRRA(const ns_rr &rr, const ns_msg &msg)
	: UpRR(rr), upa(*((in_addr*)ns_rr_rdata(rr))) { }
inline
const UpAddress& UpRRA::getAddr() const
	{ return upa; }
inline
std::ostream& operator<<(std::ostream& o, const UpRRA& urr)
	{ o << urr.name << "\tA\t" << urr.upa.c_str(); return o; }
inline
std::string& operator<<(std::string& o, const UpRRA& urr)
	{ o << urr.name << "\tA\t" << urr.upa.c_str(); return o; }
/////////////////////////////////////////////////////////////////////////////
// UpRRAAA inline methods
inline
UpRRAAAA::UpRRAAAA(const ns_rr &rr, const ns_msg &msg)
	: UpRR(rr), upa(*((in6_addr*)ns_rr_rdata(rr))) { }
inline
const UpAddress& UpRRAAAA::getAddr() const
	{ return upa; }
inline
std::ostream& operator<<(std::ostream& o, const UpRRAAAA& urr)
	{ o << urr.name << "\tAAAA\t" << urr.upa.c_str(); return o; }
inline
std::string& operator<<(std::string& o, const UpRRAAAA& urr)
	{ o << urr.name << "\tAAAA\t" << urr.upa.c_str(); return o; }
/////////////////////////////////////////////////////////////////////////////
// UpRRMX inline methods
inline
const std::string& UpRRMX::getServer() const
	{ return server; }
inline
unsigned short UpRRMX::getPreference() const
	{ return preference; }

/////////////////////////////////////////////////////////////////////////////
// UpRRSOA inline methods
inline
const std::string& UpRRSOA::getOrigin() const
	{ return origin; }
inline
const std::string& UpRRSOA::getMailAddr() const
	{ return mailAddr; }
inline
unsigned int UpRRSOA::getSerial() const
	{ return serial; }
inline
unsigned int UpRRSOA::getRefresh() const
	{ return refresh; }
inline
unsigned int UpRRSOA::getRetry() const
	{ return retry; }
inline
unsigned int UpRRSOA::getExpire() const
	{ return expire; }
inline
unsigned int UpRRSOA::getMinimum() const
	{ return minimum; }

/////////////////////////////////////////////////////////////////////////////
// UpRRNAPTR inline methods
inline
unsigned short UpRRNAPTR::getOrder() const
	{ return order; }
inline
unsigned short UpRRNAPTR::getPreference() const
	{ return preference; }
inline
const std::string& UpRRNAPTR::getFlags() const
	{ return flags; }
inline
const std::string& UpRRNAPTR::getService() const
	{ return service; }
inline
const std::string& UpRRNAPTR::getRegex() const
	{ return regex; }
inline
const std::string& UpRRNAPTR::getReplacement() const
	{ return replacement; }

/////////////////////////////////////////////////////////////////////////////
// UpRRSRV inline methods
inline
unsigned short UpRRSRV::getPreference() const
	{ return preference; }
inline
unsigned short UpRRSRV::getWeight() const
	{ return weight; }
inline
unsigned short UpRRSRV::getPort() const
	{ return port; }
inline
const std::string& UpRRSRV::getTarget() const
	{ return target; }
/////////////////////////////////////////////////////////////////////////////
// DnsQueryType inline methods
inline
void copyToUpper16(char *dst,const char *src) {
  for(int n=0; ((*src)&&(n<15)); n++ ) {
	*dst=(*src<'a') ? *src : (char)(((unsigned char)*src)&0xdf);
	src++; dst++;
  }
  *dst=0;
}
inline
void copyToLower16(char *dst,const char *src) {
  for(int n=0; ((*src)&&(n<15)); n++ ) {
	*dst=((*src>='A')&&(*src<='Z')) ? (char)(((unsigned char)*src)|0x20) : *src;
	src++; dst++;
  }
  *dst=0;
}
inline
ns_type DnsQueryType::operator()(const char* q) const {
	char buf[16]; copyToUpper16(buf,q);
	Qmap::const_iterator it=qm.find((char*)buf);
	return (it!=qm.end()) ? it->second : ns_t_invalid;
}
inline
ns_type DnsQueryType::operator()(const std::string& q) const
 { return (*this)(q.c_str()); }
inline
const char* DnsQueryType::operator()(ns_type q) const
 { return qt[q]; }
//////////////////////////////////////////////////////////////////////////////
// misc

inline
char resolverUchar2hex(unsigned char u) {
	if(u<10) return '0'+(char)u;
	if(u<16) return 'A'-10+(char)u;
	return 0;
}


#endif
