/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UPLINE_H
#define UPLINE_H

#include <UpTools/UpTimeVal.h>


#define BUFSIZE  2047
// BUFSIZE+1 ocupa un número entero de páginas de memoria
#define STATSIZE 256


class UpLine {
public:
	UpLine(int fd);
	UpLine(); // assumes reading from standard input
	void setFd(int fd); // changes file to read from (and flushes buffer)
	void flush(); // flushes buffer
	void acceptUnterminatedLines(bool v); // default: false
	void acceptLongLines(bool v); // default: false
	void acceptCRNL(bool v); // interpret '\r\n' as '\n', default: true
	void joinSeparators(bool v); //default: true
	void setTimeout(time_t t); // default timeout is infinite
	void setTimeout(const UpTimeVal& upt); // default timeout is infinite
	void setTimeout(const timeval& tv); // default timeout is infinite
	void setNoTimeout(); // this is the default: infinite wait
	const char* getStatus();
	const char* getLine();
	const char* getToken();
	char getLastSeparator();
	size_t getLen();
	int read(char* bufBytes, size_t byteCount );
	void setSeparators(const char* s, unsigned char separatorCount);
	// Default token separators:
	// NUL(\0) SOH STX ETX EOT ENQ ACK BEL(\a) BS(\b) HT(\t) LF(\n) VT(\v) FF(\f)
	// CR(\r) SO SI DLE DC1 DC2 DC3 DC4 NAK SYN ETB CAN EM SUB ESC FS GS RS US
	// SPACE , .  ; | DEL NONASCII
private:
	char buf[BUFSIZE+1];
	char statbuf[STATSIZE+1];
	int _fd,_fd1;
	char *line, *next, *endofbytes, *endofbuf;
	const char* status;
	bool _acceptUnterminatedLine; // consider eof as an endofline
	bool _fullBufIsLine; // consider full buffer as an unterminated line
	bool _joinSeparators; // consider multiple conscutive token separators as one
	bool _crnl; // consider '\r\n' as a '\n' (like in DOS text)
	size_t lineLen;
	int getBytes();
	int readWTimer(char* where, int readCount);
	fd_set rfdset,efdset,fdsetOriginal;
	UpTimeVal timeout;
	timeval timer,*timerPtr;
	char lastSeparator;
	bool separator[256];
};



#endif
