/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPLOG_INL
#define UPLOG_INL

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>

/************************* internal macros and declarations ****************/

#define LOG_THID 0x04000000

# if defined __cplusplus ? __GNUC_PREREQ (2, 6) : __GNUC_PREREQ (2, 4)
#   define __QQFUNC__  __PRETTY_FUNCTION__
# else
#  if defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L
#   define __QQFUNC__  __func__
#  else
#   define __QQFUNC__  ""
#  endif
# endif


#define _UPLOG_(level,...) { \
	if( level<=upLogUpTo ) { \
		if( upLogFd<0 ) { \
			syslog(level, __VA_ARGS__ ); \
		} else { /* log to file */ \
			char logbuf[MAXLOGLINE]; \
			int n = upLogHdr(logbuf); \
			n += snprintf(logbuf+n,MAXLOGLINE-1-n, __VA_ARGS__ ); \
			logbuf[n++] = '\n'; \
			::write(upLogFd,logbuf,n); \
			if(upLogPerror) ::write(2,logbuf,n); \
		} \
	} \
}

extern "C" {

extern int upLogPid;
extern int upLogThid;
extern int upLogPerror;
extern int upLogUpTo;
extern int upLogFd;
extern const char* upLogIdent;
extern char upLogIdentBuf[64];
extern int (*upLogHdr)(char*buf);
void upOpenLogInternal(const char *ident, int options, int facility,
                       int logLevel);
void upOpenLogFileInternal(const char* ident, int options,
                           const char* filename,int logLevel,
									int (*upLogHdrFunc)(char*));
}

#endif
