/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UPNAPTRQUERY_H
#define UPNAPTRQUERY_H

#include <UpTools/UpResolver.h>
#include <UpTools/UpInet.h>
#include <UpTools/UpSubstitute.h>

#include <list>
#include <string>


typedef enum {
	SNAPTR_NAPTR,	//1
	SNAPTR_SRV,		//2
	SNAPTR_A,		//3
	SNAPTR_SUBSTITUTE, //4
	SNAPTR_COMPLETE_OK,//5
	SNAPTR_COMPLETE_FAILED //6
} naptrState;

class UpNaptrQuery : public UpDnsQuery {

    private:

		naptrState nstate;
		std::string uri,newuri,srv,a;
		std::string service;
		std::string msg;
		unsigned short port;
      UpAddress upa;

      void completed();

    public:

		// constructor and desctructor
		UpNaptrQuery(	const std::string& _domain,
							const std::string& _uri,
							const std::string& _service );
		virtual ~UpNaptrQuery();
		// algorithm data
		naptrState			 getNStatus();
		const std::string& getMessage();
		// algorithm results
		const std::string& getSrv();
		const std::string& getA();
		UpAddress&			 getAddress();
		const std::string& getNewUri();
		unsigned short		 getPort();
		friend std::ostream& operator<<(std::ostream& o, const UpNaptrQuery& n);
		// callback to be called in derived classes
		virtual void endOfAlgorithm();
		
		
};

/////////////////////////////////////////////////////////////////////////////
// inline methods

inline UpNaptrQuery::~UpNaptrQuery()                 {  }
inline naptrState UpNaptrQuery::getNStatus()         { return nstate; }
inline UpAddress& UpNaptrQuery::getAddress()         { return upa; }
inline unsigned short UpNaptrQuery::getPort()        { return port; }
inline const std::string& UpNaptrQuery::getMessage() { return msg; }
inline const std::string& UpNaptrQuery::getNewUri()  { return newuri; }
inline const std::string& UpNaptrQuery::getA()       { return a; }
inline const std::string& UpNaptrQuery::getSrv()     { return srv; }

inline std::ostream& operator<<(std::ostream& o, const UpNaptrQuery& n)
	{ o << n.msg; return o; }

inline void UpNaptrQuery::endOfAlgorithm()
	{ }

#endif
