/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPOP_H
#define UPOP_H

#include<string>
#include<cstdlib>
#include<cstdio>
#include <UpTools/UpArch.h>


// useful functions and operators to use when handling numbers
// or strings containing numbers

bool isdigits(const std::string& s);  // true if s contains only digits
bool isnumeric(const std::string& s); // true if s contains a number (45 or -45)
bool isdigits(const char* p);         // true if p contains only digits
bool isnumeric(const char* p);        // true if p contains a number (45 or -45)

// strips leading and trailing spaced and control characters
void chomp(std::string& s);
// strips a string from spaces, tabs and control chars
void join(std::string& s);

std::string& operator<<( std::string& s , unsigned int n );
std::string& operator<<( std::string& s , int n );
std::string& operator<<( std::string& s , const std::string& q );
std::string& operator<<( std::string& s , const char* p );
std::string& operator<<( std::string& s , char c );
std::string& operator<<( std::string& s , double d );
std::string& operator<<( std::string& s , long long d );
std::string& operator<<( std::string& s , unsigned long long d );
std::string& operator<<( std::string& s , bool b );
#if __WORDSIZE==64 || defined(__s390__)
std::string& operator<<( std::string& s , size_t b );
#endif

unsigned int&       operator<<( unsigned int& n      , const std::string& s );
int&                operator<<( int& n               , const std::string& s );
double&             operator<<( double& d            , const std::string& s );
float&              operator<<( float&  d            , const std::string& s );
long long&          operator<<( long long& d         , const std::string& s );
unsigned long long& operator<<( unsigned long long& d, const std::string& s );
bool&               operator<<( bool& b              , const std::string& s );
#if __WORDSIZE==64 || defined(__s390__)
size_t&             operator<<( size_t& b              , const std::string& s );
#endif







/////////////////////////////////////////////////////////////////////////////
// inline functions

inline
std::string& operator<<( std::string& s , const std::string& q )
	{	s.append(q); return s; }

inline
std::string& operator<<( std::string& s , const char* p )
	{  s.append(p); return s; }

inline
std::string& operator<<( std::string& s , char c )
	{ s.push_back(c); return s; }



inline
bool isdigits(const std::string& s) {
	const char* p = s.c_str();
	while( *p ) if( ! std::isdigit(*p++) ) return false;
	return true;
}

inline
bool isnumeric(const std::string& s) {
	const char* p = s.c_str();
	if( *p=='-' ) p++;
	while( *p ) if( ! std::isdigit(*p++) ) return false;
	return true;
}

inline
bool isdigits(const char* p) {
	if( !p ) return false;
	while( *p ) if( ! std::isdigit(*p++) ) return false;
	return true;
}

inline
bool isnumeric(const char* p) {
	if( !p ) return false;
	if( *p=='-' ) p++;
	while( *p ) if( ! std::isdigit(*p++) ) return false;
	return true;
}

inline
std::string& operator<<( std::string& s , double d ) {
	// double precision is 52 bits or 16 decimal places
	// but %g has only 7 decimal places
	// TODO: manage 16 decimal places automatically before using scientific notation
	char buf[16]; int n=sprintf(buf,"%g",d); s.append(buf,n); return s;
}

#endif
