/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPREGEX_H
#define UPREGEX_H

#include <string>
#include <unistd.h>
#include <regex.h>
#include <iostream>
#include <vector>



class UpRegex{
public:
	UpRegex();
	UpRegex(const char* _regexptr,size_t atomCount,bool icase=false);
	UpRegex(const std::string& _regexString,size_t atomCount,bool icase=false);
	UpRegex(const UpRegex& upr); // constructor por copia
	virtual ~UpRegex();  // will be inherited by UpSubstitute

	const std::string& getRegexString();
	const std::string& getError();
	bool               status();

	bool compile( const char* _regexptr,           size_t atomCount,
					  bool icase=false );
	bool compile( const std::string& _regexString, size_t atomCount,
					  bool icase=false );
	
	bool exec(const char* data); 
	bool exec(const std::string& data);
	
	const std::string& getMatch(unsigned int index);
	const std::string& operator[](unsigned int index);

	int getMatchStart(unsigned int index);
	int getMatchEnd(unsigned int index);


protected:
	regex_t		regex;
	size_t		regexerror;
	size_t		nmatch;
	regmatch_t*	pmatch;
	int			cflags;
	std::vector<std::string> matched;
	std::string	regexString;
	std::string errorString;
	bool init;
	void statError(const char* err,size_t lnum);
	static std::string emptyString;
};



inline
bool UpRegex::exec(const std::string& data){
	return exec(data.c_str());
}


inline
bool UpRegex::status(){
	return init && !regexerror;
}

inline
const std::string& UpRegex::getRegexString() {
	return regexString;
}

inline
bool UpRegex::compile( const std::string& _regexString , size_t atomCount, bool icase ) {
	return compile(_regexString.c_str(),atomCount,icase);
}

inline
const std::string& UpRegex::operator[](unsigned int index){
	return getMatch(index);
}


#endif
