/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPROUTE_INL
#define UPROUTE_INL

// Inline methods for UpRoute

inline
void UpRoute::set(const UpAddress& baseAddr,unsigned int maskLen,unsigned int _metric) {
	m.setMask(maskLen,baseAddr.addrType); // works only for INET or INET6
	if( m.addrType!=AF_UNSPEC ) ip=baseAddr; else ip.clear();
	metric=_metric;
}

inline
void UpRoute::set(in_addr baseAddr,in_addr mask,unsigned int _metric)
{ ip=baseAddr; m = mask; metric=_metric; }


inline
void UpRoute::set(const in6_addr& baseAddr,const in6_addr& mask,unsigned int _metric)
{ ip=baseAddr; m = mask; metric=_metric; }

inline
void UpRoute::set(const UpAddress& baseAddr,const UpAddress& mask,unsigned int _metric) {
	if( baseAddr.family()!=mask.family() or
		!(mask.family()==AF_INET or mask.family()==AF_INET6) )
	{ ip.clear(); m.clear(); }
 	else { ip=baseAddr; m=mask; }
	metric=_metric;
}

inline
void UpRoute::set(const UpRoute& r)
{ ip=r.ip; m=r.m; metric=r.metric; }

inline
void UpRoute::set(const UpNet& n,unsigned int _metric)
{ ip=n.ip; m=n.m; metric=_metric; }

inline
bool UpRoute::remove(const UpAddress& _ip,unsigned int _m,bool includeSubnets) {
	UpAddress M;M.setMask(_m,_ip.family()); return remove(_ip,M,includeSubnets);
}

inline
void UpRoute::append(std::ostream& o) const
{ o<<ip<<'/'<<m.maskBitsLen()<<' '<<metric; }

inline
void UpRoute::append(std::string& o) const
{ o<<ip<<'/'<<m.maskBitsLen()<<' '<<metric; }

inline
void UpRoute::setMetric(unsigned int _metric)
{	metric = _metric; }

inline
std::ostream& operator<<(std::ostream& o,const UpRoute& net)
{ net.append(o); return o; }

inline
std::string& operator<<(std::string& o,const UpRoute& net)
{ net.append(o); return o; }

inline
const UpRoute* UpRoute::lookup(const UpSockAddr& sa,bool useMetric) const {
	switch( sa.family() ) {
		case AF_INET:  return lookup(sa.sa.in->sin_addr);
		case AF_INET6: return lookup(sa.sa.in6->sin6_addr);
		default: return 0;
	}
}

inline
const UpRoute* UpRoute::lookup(const UpAddress& ipAddr,bool useMetric) const {
	switch( ipAddr.family() ) {
		case AF_INET:  return lookup(ipAddr.upAddr.a4,useMetric);
		case AF_INET6: return lookup(ipAddr.upAddr.a6,useMetric);
		default: return 0;
	}
}

inline
UpRoute& UpRoute::operator=(const std::string& s)
{ set(s); return *this; }


#endif
