/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// inline code for UpSmartLinkPtr and UpLinkable
#ifndef SMARTLINKPTR_INL
#define SMARTLINKPTR_INL

#if ( __GNUC__ < 4) || ( ( __GNUC__ == 4 ) && ( __GNUC_MINOR__ < 2 ))
# include <bits/atomicity.h>
#else
# include <ext/atomicity.h>
#endif
#include<UpTools/UpLog.h>
#include<UpTools/UpArch.h>
#include <typeinfo>



#ifdef UPLINKABLE_LOG_EVENTS

	#define UPLINKABLE_MONITOR 1

#endif





#ifdef UPLINKABLE_MONITOR

	#ifdef UPLINKABLE_LOG_EVENTS

		#define MONITOR_UNNAMED_CONSTRUCTOR_EVENT \
			unsigned int upl( upLinkableMonitor.constructorEvent() ); \
			UPLOG(LOG_DEBUG,"Unnamed UpLinkable 0x"PFX" constructor, population:" \
				"%u, (it is better to use named constructor)", \
            PTR2PRINTFCAST(this),upl);

	
		#define MONITOR_NAMED_CONSTRUCTOR_EVENT \
			unsigned int upl( upLinkableMonitor.constructorEvent(className) ); \
			UPLOG(LOG_DEBUG,"%s 0x"PFX" constructor, population:%u ", \
			  (className?className:"Unnamed UpLinkable"),PTR2PRINTFCAST(this),upl);

	
		#define MONITOR_DESTRUCTOR_EVENT \
			unsigned int upl( upLinkableMonitor.destructorEvent(className) ); \
			UPLOG(LOG_DEBUG,"%s 0x"PFX" destructor, remaining population:%u", \
				(className?className:"Unnamed UpLinkable"),PTR2PRINTFCAST(this),upl);


	#else

		#define MONITOR_UNNAMED_CONSTRUCTOR_EVENT \
			upLinkableMonitor.constructorEvent();
	
		#define MONITOR_NAMED_CONSTRUCTOR_EVENT \
			upLinkableMonitor.constructorEvent(className);
	
		#define MONITOR_DESTRUCTOR_EVENT \
			upLinkableMonitor.destructorEvent(className);

	#endif

#else

	#define MONITOR_UNNAMED_CONSTRUCTOR_EVENT

	#define MONITOR_NAMED_CONSTRUCTOR_EVENT

	#define MONITOR_DESTRUCTOR_EVENT

#endif


// if distance between pointer to the object and pointer to data in
// constructor stack is less than STACK_DATA_SAFE_DISTANCE, the object
// is considered to be in stack instead of the heap
#define STACK_DATA_SAFE_DISTANCE 0xfffff



#ifdef UPLINKABLE_HEAPCHECK

	#define MONITOR_HEAPCHECK \
		/* create a buffer in stack (not in heap, nor in register) */  \
		char dummybuf[16]; \
		/* calulate distance between buffer and the pointer to this */ \
		register size_t distanceToStack( \
			((char*)dummybuf>(char*)this) ? \
			((char*)dummybuf-(char*)this) : ((char*)this-(char*)dummybuf) \
		); \
		/* consider this to be in the heap if distance to stack buffer is big */ \
		if( distanceToStack < STACK_DATA_SAFE_DISTANCE ) { \
			UPLOG(LOG_ERR,"%s constructor error: distance to stack="PFU", this " \
				"class must only be instantiated in the heap! (use new)", \
				(className?className:"UpLinkable"), \
				SIZET2PRINTFCAST(distanceToStack) ); \
			exit(1); \
		}

#else

	#define MONITOR_HEAPCHECK

#endif		





#ifdef UPLINKABLE_LOG_LINKS

	#define MONITOR_LINK_EVENT { \
		const char* pn(typeid(T).name()); \
		if(pn!=0) { while(*pn>='0' && *pn<='9') ++pn; } else pn=""; \
		const char* psp(sp->name()); \
		if(psp==0) psp="unnamed UpLinkable"; \
		UPLOG(LOG_DEBUG,"UpSmartLinkPtr<%s> 0x"PFX" linking %s 0x"PFX \
				" count:%u", pn, PTR2PRINTFCAST(this), psp, PTR2PRINTFCAST(sp), \
				sp->count() ); \
		}

	#define MONITOR_UNLINK_EVENT { \
		const char* pn(typeid(T).name()); \
		if(pn!=0) { while(*pn>='0' && *pn<='9') ++pn; } else pn=""; \
		const char* psp(sp->name()); \
		if(psp==0) psp="unnamed UpLinkable"; \
		UPLOG(LOG_DEBUG,"UpSmartLinkPtr<%s> 0x"PFX" UNlinking %s 0x"PFX \
				" count:%u", pn, PTR2PRINTFCAST(this), psp, PTR2PRINTFCAST(sp),\
				count ); \
		}

#else

	#define MONITOR_LINK_EVENT

	#define MONITOR_UNLINK_EVENT

#endif



/////////////////////////////////////////////////////////////////////////////
// UpLinkable inline methods
/////////////////////////////////////////////////////////////////////////////

inline void UpLinkable::link() const
{ __gnu_cxx::__atomic_add(&__count,1); }

inline int  UpLinkable::unlink() const
{ return __gnu_cxx::__exchange_and_add(&__count,-1)-1; }

inline int  UpLinkable::unlinkAndReturnPriorValue() const
{ return __gnu_cxx::__exchange_and_add(&__count,-1); }


inline int  UpLinkable::count() const
{ return __count; }


inline const char* UpLinkable::name() const
{ return className; }



inline UpLinkable::UpLinkable()
: __count(0), className(0)
{
	MONITOR_HEAPCHECK
	MONITOR_UNNAMED_CONSTRUCTOR_EVENT
}



inline UpLinkable::UpLinkable(const char* _className)
: __count(0), className(_className)
{
	MONITOR_HEAPCHECK
	MONITOR_NAMED_CONSTRUCTOR_EVENT
}




inline UpLinkable::UpLinkable(const UpLinkable& ll)
: __count(0), className(ll.className) // copy constructor also sets __count=0
{
	MONITOR_HEAPCHECK
	MONITOR_NAMED_CONSTRUCTOR_EVENT
}




inline UpLinkable::~UpLinkable() {
	MONITOR_DESTRUCTOR_EVENT
}




/////////////////////////////////////////////////////////////////////////////
// UpSmartLinkPtr<T> inline methods
/////////////////////////////////////////////////////////////////////////////
template<typename T>
inline UpSmartLinkPtr<T>::UpSmartLinkPtr()
: sp(0)
{ }

template<typename T>
inline UpSmartLinkPtr<T>::UpSmartLinkPtr(const T* ptr)
: sp((T*)ptr)
{ hold(); }

template<typename T>
inline UpSmartLinkPtr<T>::UpSmartLinkPtr(const UpSmartLinkPtr<T>& sptr)
: sp((T*)(sptr.sp))
{ hold(); } 

// destructor
template<typename T>
inline UpSmartLinkPtr<T>::~UpSmartLinkPtr()
{ release(); }

// operadores de asignacion
template<typename T>
inline UpSmartLinkPtr<T>& UpSmartLinkPtr<T>::operator=(const T* ptr) {
	if(ptr==sp) return *this;
	release(); sp=(T*)ptr; hold(); return *this;
}

template<typename T>
inline UpSmartLinkPtr<T>& UpSmartLinkPtr<T>::operator=(const UpSmartLinkPtr<T>& sptr) {
	if(sptr.sp==sp) return *this;
	release(); sp=(T*)(sptr.sp); hold(); return *this;
}

// cast to void* operator
template<typename T>
inline UpSmartLinkPtr<T>::operator void*() { return sp; } 

// operadores utiles
template<typename T>
inline T& UpSmartLinkPtr<T>::operator*()
	{ return *sp; }

template<typename T>
inline const T& UpSmartLinkPtr<T>::operator*() const
	{ return *sp; }

template<typename T>
inline T* UpSmartLinkPtr<T>::operator->()
	{ return  sp; }

template<typename T>
inline const T* UpSmartLinkPtr<T>::operator->() const
	{ return  sp; }

template<typename T>
inline bool	UpSmartLinkPtr<T>::operator!() const
	{ return sp ? false : true; }

// operadores de comparacion
template<typename T>
inline bool	UpSmartLinkPtr<T>::operator==(const T* ptr) const
	{ return sp==(T*)ptr; }  

template<typename T>
inline bool	UpSmartLinkPtr<T>::operator!=(const T* ptr) const
	{ return sp!=(T*)ptr; }

template<typename T>
inline bool	UpSmartLinkPtr<T>::operator==(const UpSmartLinkPtr<T>& sptr) const
	{ return sp==(T*)(sptr.sp); }

template<typename T>
inline bool	UpSmartLinkPtr<T>::operator!=(const UpSmartLinkPtr<T>& sptr) const
	{ return sp!=(T*)(sptr.sp); }

template<typename T>
inline bool UpSmartLinkPtr<T>::operator<(const UpSmartLinkPtr<T>& sptr) const
	{ return sp<(T*)(sptr.sp); }

template<typename T>
inline void UpSmartLinkPtr<T>::hold() {
	if(sp) {
		sp->link();
		MONITOR_LINK_EVENT	
	}
}

template<typename T>
inline void UpSmartLinkPtr<T>::release() {
	if(sp) {
		#ifdef UPLINKABLE_LOG_LINKS
		int count=sp->unlink();
		MONITOR_UNLINK_EVENT
		if( count==0 ) delete sp;
		#else
		if( sp->unlinkAndReturnPriorValue()==1 )
			delete sp;
		#endif
	}
}

#endif

