/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

// internal sql classes not to be used by library users

#ifndef UPCORE_SQL_INL
#define UPCORE_SQL_INL

#include <UpTools/UpOp.h>


//////////////////////////////////////////////////////////////////////////////
// clasas QueryElem
// this class is going to be inherited by specific DB query implementations

class QueryElem {
public:
	QueryElem(unsigned int _fCount);
	void		deleteNext();				// recursivo
	void		pushBack(QueryElem* q);	// recursivo
   virtual	~QueryElem();				// no recursivo 
	virtual unsigned long long	lastInsertId(); // some DB's do not support it!
	virtual unsigned long long	rowCount();
	virtual bool 					fetchRow();
	virtual const char*			col(unsigned int c);
	virtual int						colInt(unsigned int c);
	virtual long long				colLongLong(unsigned int c);
	virtual double					colDouble(unsigned int c);
	virtual bool					colBool(unsigned int c);
protected:
	friend class UpSqlQuery;
	unsigned int fCount;
	QueryElem*   next;
};




/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// inline code for QueryElem
inline
QueryElem::QueryElem(unsigned int _fCount)
: fCount(_fCount), next(0)
{ }

inline
QueryElem::~QueryElem() // no recursivo 
{ }

inline
void QueryElem::deleteNext() // recursivo
{ if(next) { next->deleteNext(); delete next;  } }

inline
void QueryElem::pushBack(QueryElem* q) // recursivo
{  if(next) next->pushBack(q); else next = q; }

inline
unsigned long long QueryElem::lastInsertId()
{ return 0; } // default for DB's that do not support last insert id

inline
unsigned long long QueryElem::rowCount()
{ return 0; }

inline
bool  QueryElem::fetchRow()
{ return false; }

inline
const char* QueryElem::col(unsigned int c)
{ return ""; }

inline
int QueryElem::colInt(unsigned int c)
{ return 0; }

inline
long long QueryElem::colLongLong(unsigned int c)
{ return 0; }

inline
double QueryElem::colDouble(unsigned int c)
{ return 0.0; }

inline
bool QueryElem::colBool(unsigned int c)
{ return false; }

//////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// inline code for UpSqlConn
inline
UpSqlConn::~UpSqlConn()
{ }

inline
bool UpSqlConn::status()
{ return statusOk; }

inline
const char* UpSqlConn::errorPhrase()
{ return statusOk ? "ok" : statusPhrase; }

//////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// inline code for UpSqlQuery
inline
unsigned int UpSqlQuery::errorNumber()
{ return errNumber; }

inline
const char* UpSqlQuery::errorPhrase()
{ return errPhrase; }

inline
unsigned int UpSqlQuery::fieldCount()
{ return fCount; }

inline
unsigned long long UpSqlQuery::lastInsertId()
{ return elem->lastInsertId(); }

inline
unsigned long long UpSqlQuery::rowCount()
{ return elem->rowCount(); }

inline
bool UpSqlQuery::fetchRow()
{ return elem->fetchRow(); }

inline
const char* UpSqlQuery::col(unsigned int c)
{ return elem->col(c); }

inline
int UpSqlQuery::colInt(unsigned int c) 
{ return elem->colInt(c); }


inline
long long  UpSqlQuery::colLongLong(unsigned int c)
{ return elem->colLongLong(c); }


inline
double UpSqlQuery::colDouble(unsigned int c)
{ return elem->colDouble(c); }


inline
bool UpSqlQuery::colBool(unsigned int c)
{ return elem->colBool(c); }


/////////////////////////////////////////////////////////////////////////////
//class UpSqlConnUndef

class UpSqlConnUndef : public UpSqlConn {
public:
	UpSqlConnUndef()
		: UpSqlConn(DBTYPE_UNDEF) {}
	virtual void disconnect()
		{ }
	virtual bool exec(UpSqlQuery& q)
		{ return false; }
	virtual bool exec(UpSqlQuery& q,const char* sqlText)
		{ return false; }
	virtual bool exec(UpSqlQuery& q,const char* sqlText,size_t sqlTextLen)
		{ return false; }
	virtual bool privateConnect(const std::string& host)
		{ return false; }
	virtual bool errorIsConnectionError(unsigned int err)
		{ return false; }
	virtual UpSqlConn* newCopyConn() const
		{ return new UpSqlConnUndef; }
};

extern UpSqlConnUndef upSqlConnUndef;
#endif 
