/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPCORE_SQLMYSQL_INL
#define UPCORE_SQLMYSQL_INL

#include <cstring>

//////////////////////////////////////////////////////////////////////////////
// UpSqlConnMysql inline methods

inline
UpSqlConnMysql::UpSqlConnMysql()
: UpSqlConn(DBTYPE_MYSQL)
{ }

inline
UpSqlConnMysql::UpSqlConnMysql(const UpSqlConn& c)
: UpSqlConn(c)
{ }

inline
UpSqlConnMysql::UpSqlConnMysql(const UpSqlConnMysql& c)
: UpSqlConn(c)
{ }

inline
UpSqlConnMysql::~UpSqlConnMysql()
{ if(need_close) mysql_close(&conn); }

inline
bool UpSqlConnMysql::exec(UpSqlQuery& q,const char* sqlText)
{ return exec( q, sqlText, std::strlen(sqlText) ); }

inline
bool UpSqlConnMysql::exec(UpSqlQuery& q)
{ return exec( q, q.sql.c_str(), q.sql.size() ); }

inline
UpSqlConn* UpSqlConnMysql::newCopyConn() const
{ return new UpSqlConnMysql(*this); }

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// class QueryElemMysql and inline methods

class QueryElemMysql : public QueryElem {
public:
	QueryElemMysql(MYSQL_RES* _res,unsigned int _fCount,my_ulonglong _lastId);
    ~QueryElemMysql();
	unsigned long long	lastInsertId(); // supported by mysql
	unsigned long long	rowCount();
	bool 						fetchRow();
	const char*				col(unsigned int c);
	int						colInt(unsigned int c);
	long long				colLongLong(unsigned int c);
	double					colDouble(unsigned int c);
	bool						colBool(unsigned int c);
private:
	friend class UpSqlQuery;
	MYSQL_RES*   res;
	MYSQL_ROW	 row;
	my_ulonglong lastId;
};

/////////////////////////////////////////////////////////////////////////////

inline
QueryElemMysql::QueryElemMysql(
		MYSQL_RES* _res,unsigned int _fCount,my_ulonglong _lastId)
: QueryElem(_fCount), res(_res), row(0), lastId(_lastId)
{ }

inline
QueryElemMysql::~QueryElemMysql() // no recursivo 
{ if(res) mysql_free_result(res); }

inline
unsigned long long QueryElemMysql::rowCount()
{ return res ? mysql_num_rows(res) : 0; }

inline
bool QueryElemMysql::fetchRow()
{ if( res==0 ) return false; row=mysql_fetch_row(res); return (row!=0); }

inline
unsigned long long QueryElemMysql::lastInsertId()
{ return lastId; }

inline
const char* QueryElemMysql::col(unsigned int c)
{ return  ( row && (c<fCount) && row[c] ) ? row[c] : "" ; }

inline
int QueryElemMysql::colInt(unsigned int c)
{ return ( row && (c<fCount) && row[c] ) ? std::atoi(row[c]) : 0 ; }

inline
long long QueryElemMysql::colLongLong(unsigned int c)
{ return ( row && (c<fCount) && row[c] ) ? std::atoll(row[c]) : 0 ; }

inline
double QueryElemMysql::colDouble(unsigned int c)
{ return ( row && (c<fCount) && row[c] ) ? std::atof(row[c]) : 0 ; }

inline // revisar que pasa cuando el campo es bool en la base
bool QueryElemMysql::colBool(unsigned int c)
{ return ( row && (c<fCount) && row[c] ) ? (*row[c]!='0') : false ; }

#endif 
