/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPCORE_SQLPGSQL_H
#define UPCORE_SQLPGSQL_H

#include <UpTools/UpSql.h>
#include <libpq-fe.h>

class UpSqlConnPgsql : public UpSqlConn {
public:
	UpSqlConnPgsql();
	UpSqlConnPgsql(const UpSqlConnPgsql&);
	UpSqlConnPgsql(const UpSqlConn&);
	~UpSqlConnPgsql();

	// see UpTools/UpSql.h for inherited methods

	// Close connection to DB. It is automatically done in destructor.
	virtual void disconnect();

	// execute query and store results (there may be many results if there
	// are more than one sql sentence in query)
	bool exec(UpSqlQuery& q);
	bool exec(UpSqlQuery& q,const char* sqlText);
	bool exec(UpSqlQuery& q,const char* sqlText,size_t sqlTextLen);

private:
	PGconn *conn;
	bool privateConnect(const std::string& host);
	bool errorIsConnectionError(unsigned int err);
	friend class UpSqlConnPool;
   UpSqlConn* newCopyConn() const;
};



#include <UpTools/UpSqlPgsql.inl>

#endif
