/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPCORE_SQLPGSQL_INL
#define UPCORE_SQLPGSQL_INL

inline
UpSqlConnPgsql::UpSqlConnPgsql()
: UpSqlConn(DBTYPE_PGSQL), conn(0)
{ }

inline
UpSqlConnPgsql::UpSqlConnPgsql(const UpSqlConn& c)
: UpSqlConn(c), conn(0)
{ }

inline
UpSqlConnPgsql::UpSqlConnPgsql(const UpSqlConnPgsql& c)
: UpSqlConn(c), conn(0)
{ }

inline
UpSqlConnPgsql::~UpSqlConnPgsql()
{ if(need_close) PQfinish(conn); }

inline
bool UpSqlConnPgsql::exec(UpSqlQuery& q,const char* sqlText,size_t sqlTextLen)
{
	if( sqlText[sqlTextLen]!=0 ) // we need a null terminated query in postgresql!
		{ q.sql.assign(sqlText,sqlTextLen); return exec( q, q.sql.c_str() ); }
	return exec( q, sqlText );
}

inline
bool UpSqlConnPgsql::exec(UpSqlQuery& q)
{ return exec( q, q.sql.c_str() ); }

inline
UpSqlConn* UpSqlConnPgsql::newCopyConn() const
{ return new UpSqlConnPgsql(*this); }

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

class QueryElemPgsql : public QueryElem {
public:
	QueryElemPgsql(PGresult* _res);
	~QueryElemPgsql();
	unsigned long long	rowCount();
	bool 						fetchRow();
	const char*				col(unsigned int c);
	int						colInt(unsigned int c);
	long long				colLongLong(unsigned int c);
	double					colDouble(unsigned int c);
	bool						colBool(unsigned int c);
private:
	friend class UpSqlQuery;
	PGresult*   res;
	int			r;
	int			rCount;
};

//////////////////////////////////////////////////////////////////////////////

inline
QueryElemPgsql::QueryElemPgsql(PGresult* _res)
: QueryElem((unsigned int)PQnfields(_res)),
  res(_res), r(-1), rCount(PQntuples(_res))
{ }

inline
QueryElemPgsql::~QueryElemPgsql() // no recursivo 
{ if(res) PQclear(res); }

inline
unsigned long long QueryElemPgsql::rowCount()
{ return rCount; }

inline
bool QueryElemPgsql::fetchRow()
{ return ( ++r<rCount ); }

#define checkRC() \
	( ((unsigned int)r<(unsigned int)rCount) and  (c<fCount) )


inline
const char* QueryElemPgsql::col(unsigned int c)
{ return checkRC() ? PQgetvalue(res,r,c) : ""; }

inline
int QueryElemPgsql::colInt(unsigned int c)
{ return checkRC() ? std::atoi(PQgetvalue(res,r,c)) : 0; }

inline
long long QueryElemPgsql::colLongLong(unsigned int c)
{ return checkRC() ? std::atoll(PQgetvalue(res,r,c)) : 0; }

inline
double QueryElemPgsql::colDouble(unsigned int c)
{ return checkRC() ? std::atof(PQgetvalue(res,r,c)) : 0.0; }

inline // revisar que pasa cuando el campo es bool en la base
bool QueryElemPgsql::colBool(unsigned int c)
{ if( ! checkRC() ) return false;
  char val(*PQgetvalue(res,r,c));
  return val=='t' or val=='1';
}

#endif 
