/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef UPSUBSTITUTE_H
#define UPSUBSTITUTE_H

#include <UpTools/UpRegex.h>
#include <UpTools/UpLog.h>
#include <string>

class UpSubstitute : public UpRegex {
	private:
	    typedef enum { SC_REGEX, SC_BRACKET, SC_BRACKET_FIRST, SC_ATOM,
						SC_ESCAPED, SC_SUBS,SC_FLAGS } scanState;
		bool icase;
		std::string combinedExpression; // contiene regex y substituteExpression
		std::string substituteExpression;
		std::string result; // la expresion con las substituciones
		// static data (modifiable by application)
		static char substituteIndicator;  // caracter indicador de substitucion
		// private methods
		bool combinedExpressionScanner(
					const std::string& _regexAndSubstituteExpression );
		bool regexScanner(
					const std::string& _regexExpression );


	public:
		UpSubstitute();
		~UpSubstitute();
		UpSubstitute(const UpSubstitute&);// constructor por copia
		// A) compilacion a partir de datos individuales ingresados por
		//    el usuario
		UpSubstitute(const std::string& _regex,
					 const std::string& _substituteExpression,
					 size_t _atomCount,bool _icase=false);
		bool compile(const std::string& _regex,
					 const std::string& _substituteExpression,
					 size_t _atomCount,bool _icase=false);
		// B) calcula solo los parentesis
		// ( si la expresion regular esta disponible en el momento de la
		//   programacion, entonces es mejor usar los metodos A porque son mas
		// rapidos)
		UpSubstitute(const std::string& _regex,
					 const std::string& _substituteExpression,
					 bool _icase=false);
		bool compile(const std::string& _regex,
					 const std::string& _substituteExpression,
					 bool _icase=false);
		// C) compilacion a partir de una unica expresion como ser:
		//        /regex/substituteExpression/i
		//     o  !regex!substituteExpression!
		UpSubstitute(const std::string& _regexAndSubstituteExpression);
		bool compile(const std::string& _regexAndSubstituteExpression);

		// ejecucion y resultados
		bool exec(const std::string& _text);
		const std::string& getResult();
};

inline
UpSubstitute::UpSubstitute() { }
inline
UpSubstitute::~UpSubstitute() { }
inline
const std::string& UpSubstitute::getResult() { return result; }

#endif
