/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef UPTHREAD_INL
#define UPTHREAD_INL

///////////////////////////////////////////////////////////////////////////////
// UpMutex Methods ////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
inline
UpMutex::UpMutex(){
	pthread_mutex_init(&mutex,0);
}

inline
UpMutex::UpMutex(int kind){
	pthread_mutexattr_t attr; pthread_mutexattr_init(&attr);
	pthread_mutexattr_settype(&attr,kind);
		pthread_mutex_init(&mutex,&attr);
	pthread_mutexattr_destroy(&attr);
}

inline
UpMutex::~UpMutex () { pthread_mutex_destroy(&mutex); }

inline
int UpMutex::lock() { return pthread_mutex_lock(&mutex); }

inline
int UpMutex::unlock() { return pthread_mutex_unlock(&mutex); }

inline
int UpMutex::trylock() { return pthread_mutex_trylock(&mutex); }


/////////////////////////////////////////////////////////////////////////////
// UpScopedLock methods /////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
inline
UpScopedLock::UpScopedLock(UpMutex& _mx) : mx(_mx) { mx.lock(); }

inline
UpScopedLock::~UpScopedLock() { mx.unlock(); }
/////////////////////////////////////////////////////////////////////////////
// UpSemaphore methods //////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
inline
UpSemaphore::UpSemaphore(unsigned int initialValue) {
	sem_init( &s , 0 /* only for thread-shared semaphores */, initialValue );
}

inline
UpSemaphore::~UpSemaphore()	{ sem_destroy(&s); }

inline
int UpSemaphore::wait()		{ return sem_wait(&s); }

inline
int UpSemaphore::post()		{ return sem_post(&s); }

inline
int UpSemaphore::getValue()	{ int n; sem_getvalue(&s,&n); return n; }

///////////////////////////////////////////////////////////////////////////////
// UpRWLock Methods ////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
inline
UpRWLock::UpRWLock() { pthread_rwlock_init(&rwl,0); }

inline
UpRWLock::~UpRWLock () { pthread_rwlock_destroy(&rwl); }

inline
int UpRWLock::RDLock() { return pthread_rwlock_rdlock(&rwl); }

inline
int UpRWLock::tryRDLock() { return pthread_rwlock_tryrdlock(&rwl); }

inline
int UpRWLock::WRLock() { return pthread_rwlock_wrlock(&rwl); }

inline
int UpRWLock::tryWRLock() { return pthread_rwlock_trywrlock(&rwl); }

inline
int UpRWLock::unlock() { return pthread_rwlock_unlock(&rwl); }

#endif
