/* UpTools v8.6
 *
 * Copyright (c) 2005-2012 Fundacion Universidad de Palermo (Argentina).
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment: 'This product includes software developed by the
 *    "Universidad de Palermo, Argentina" (http://www.palermo.edu/).'
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/////////////////////////////////////////////////////////////////////////////
// UpTimeVal Methods

inline
UpTimeVal&  UpTimeVal::operator=(const std::string& s) {
	return (*this)=s.c_str();
}

inline
UpTimeVal&  UpTimeVal::operator=(const char* p) {
	clear();
	if( p==0 || *p==0 ) return *this;
	register char c;
	for(;;) {
		c=*p;
		if( c<'0'|| c>'9') break;
		tv_sec*=10; tv_sec+=(time_t)(c-'0');
		p++;
	}
	if(c=='.') {
		p++;
		const char* q=p;
		for(;;) {
			c=*p;
			if( c<'0'|| c>'9') break;
			tv_usec*=10; tv_usec+=(time_t)(c-'0');
			if((p-q)>=5) break; // max 6 digits
			p++;
		}
	}
	return *this;
}


inline
std::ostream& operator<<(std::ostream& o,const UpTimeVal& t){
	if( t.tv_sec<0 and t.tv_usec>0 ) {
		UpTimeVal aux(-1-t.tv_sec,1000000-t.tv_usec);
		o<<'-';
		UpTimeVal::basicCout(o,aux);
	} else {
		UpTimeVal::basicCout(o,t);
	}
	return o;
}

inline
std::string& operator<<(std::string& s,const UpTimeVal& t){
	if( t.tv_sec<0 and t.tv_usec>0 ) {
		UpTimeVal aux(-1-t.tv_sec,1000000-t.tv_usec);
		s.append(1,'-');
		UpTimeVal::basicSout(s,aux);
	} else {
		UpTimeVal::basicSout(s,t);
	}
	return s;
}
//////////////////////////////////////////////////////////////////////////////
inline UpTimeVal::UpTimeVal()
: tv_sec(0), tv_usec(0)
{ }

inline UpTimeVal::UpTimeVal(double d) { // argument in seconds
   register double fl=floor(d);
   tv_sec=(time_t)fl; tv_usec=(suseconds_t)(1000000.0*(d-fl));
}


inline UpTimeVal::UpTimeVal(time_t s,suseconds_t u)
: tv_sec(s), tv_usec(u)
{ }

inline UpTimeVal::UpTimeVal(const UpTimeVal& t)
: tv_sec(t.tv_sec), tv_usec(t.tv_usec) 
{ }

inline UpTimeVal::UpTimeVal(const timeval& t)
: tv_sec(t.tv_sec), tv_usec(t.tv_usec)
{ }

inline void UpTimeVal::clear() {
	tv_sec=tv_usec=0;
}

inline bool UpTimeVal::isset() const {
	return tv_sec || tv_usec;
}

inline void UpTimeVal::now() {
	gettimeofday((timeval*)&tv_sec,NULL);
}

inline const UpTimeVal& UpTimeVal::tic() {
	gettimeofday((timeval*)&tv_sec,NULL);
	return *this;
}

inline const UpTimeVal& UpTimeVal::toc() {
	UpTimeVal c;
	gettimeofday((timeval*)&(c.tv_sec),NULL);
   c.tv_sec  -= tv_sec;
   c.tv_usec -= tv_usec;
   if( c.tv_usec < 0 ) {
      c.tv_usec += 1000000;
      (c.tv_sec)--;
   }
	tv_sec  = c.tv_sec;
	tv_usec = c.tv_usec;
	return *this;
}

/////////////////////////////////////////////////////////////////////////////
inline void UpTimeVal::charDate(std::string& s) const {
	s.resize(48); s.resize( charDate((char*)s.data(),s.size()) );
}

inline void UpTimeVal::charTime(std::string& s) const {
	s.resize(16); s.resize( charTime((char*)s.data(),s.size()) );
}

inline void UpTimeVal::charDateTime(std::string& s) const {
	s.resize(64); s.resize( charDateTime((char*)s.data(),s.size()) );
}
/////////////////////////////////////////////////////////////////////////////
inline bool UpTimeVal::readDate(const std::string& s) {
	return readDate(s.c_str());
}

inline bool UpTimeVal::readDateTime(const std::string& s) {
	return readDateTime(s.c_str());
}

/////////////////////////////////////////////////////////////////////////////
inline UpTimeVal& UpTimeVal::operator=(const UpTimeVal& t) {
	tv_sec=t.tv_sec; tv_usec=t.tv_usec;
	return *this;
}


inline UpTimeVal UpTimeVal::operator+(const UpTimeVal& a) const {
	UpTimeVal c;
	c.tv_sec  = a.tv_sec  + tv_sec;
	c.tv_usec = a.tv_usec + tv_usec;
if( c.tv_usec > 999999) {
		c.tv_usec -= 1000000;
		(c.tv_sec)++;
	}
	return c;
}

inline UpTimeVal UpTimeVal::operator-(const UpTimeVal& a) const {
	UpTimeVal c;
	c.tv_sec  = tv_sec  - a.tv_sec;
	c.tv_usec = tv_usec - a.tv_usec;
	if( c.tv_usec < 0 ) {
		c.tv_usec += 1000000;
		(c.tv_sec)--; 
	}
	return c;
}

inline bool UpTimeVal::operator>(const UpTimeVal& a) const {
	return tv_sec>a.tv_sec || (tv_sec==a.tv_sec && tv_usec>a.tv_usec);
}

inline bool UpTimeVal::operator<(const UpTimeVal& a) const {
	return tv_sec<a.tv_sec || (tv_sec==a.tv_sec && tv_usec<a.tv_usec);
}

inline bool UpTimeVal::operator==(const UpTimeVal& a) const {
	return tv_sec==a.tv_sec && tv_usec==a.tv_usec;
}

inline bool UpTimeVal::operator!=(const UpTimeVal& a) const {
	return tv_sec!=a.tv_sec || tv_usec != a.tv_usec;
}

inline double UpTimeVal::operator/(const UpTimeVal& a) const {
	return ( 1000000.0*(double)tv_sec + (double)tv_usec ) /
			 ( 1000000.0*(double)a.tv_sec + (double)a.tv_usec );
}

inline UpTimeVal& UpTimeVal::operator+=(const UpTimeVal& a) {
	tv_sec  = a.tv_sec  + tv_sec;
	tv_usec = a.tv_usec + tv_usec;
	if( tv_usec > 999999) {
		tv_usec -= 1000000;
		tv_sec++;
	}
	return *this;
}

inline UpTimeVal& UpTimeVal::operator-=(const UpTimeVal& a) {
	tv_sec  -= a.tv_sec;
	tv_usec -= a.tv_usec;
	if( tv_usec < 0 ) {
		tv_usec += 1000000;
		--tv_sec;
	}
	return *this;
}

/////////////////////////////////////////////////////////////////////////////
inline UpTimeVal& UpTimeVal::operator=(const timeval& t) {
	tv_sec=t.tv_sec; tv_usec=t.tv_usec;
 	return *this;
}

inline UpTimeVal UpTimeVal::operator+(const timeval& a) const {
	UpTimeVal c;
	c.tv_sec  = a.tv_sec  + tv_sec;
	c.tv_usec = a.tv_usec + tv_usec;
	if( c.tv_usec > 999999) {
		c.tv_usec -= 1000000;
		(c.tv_sec)++;
	}
	return c;
}

inline UpTimeVal UpTimeVal::operator-(const timeval& a) const {
	UpTimeVal c;
	c.tv_sec  = tv_sec  - a.tv_sec;
	c.tv_usec = tv_usec - a.tv_usec;
	if( c.tv_usec < 0 ) {
		c.tv_usec += 1000000;
		(c.tv_sec)--; 
	}
	return c;
}

inline bool UpTimeVal::operator>(const timeval& a) const {
	return tv_sec>a.tv_sec || (tv_sec==a.tv_sec && tv_usec>a.tv_usec);
}

inline bool UpTimeVal::operator<(const timeval& a) const {
	return tv_sec<a.tv_sec || (tv_sec==a.tv_sec && tv_usec<a.tv_usec);
}

inline bool UpTimeVal::operator==(const timeval& a) const {
	return tv_sec==a.tv_sec && tv_usec==a.tv_usec;
}

inline bool UpTimeVal::operator!=(const timeval& a) const {
	return tv_sec!=a.tv_sec || tv_usec != a.tv_usec;
}

inline double UpTimeVal::operator/(const timeval& a) const {
	return ( 1000000.0*(double)tv_sec + (double)tv_usec ) /
			 ( 1000000.0*(double)a.tv_sec + (double)a.tv_usec );
}

inline UpTimeVal& UpTimeVal::operator+=(const timeval& a) {
	tv_sec  = a.tv_sec  + tv_sec;
	tv_usec = a.tv_usec + tv_usec;
	if( tv_usec > 999999) {
		tv_usec -= 1000000;
		tv_sec++;
	}
	return *this;
}

inline UpTimeVal& UpTimeVal::operator-=(const timeval& a) {
	tv_sec  -= a.tv_sec;
	tv_usec -= a.tv_usec;
	if( tv_usec < 0 ) {
		tv_usec += 1000000;
		--tv_sec;
	}
	return *this;
}

/////////////////////////////////////////////////////////////////////////////
inline UpTimeVal& UpTimeVal::operator=(time_t t) { // argument in seconds
	tv_sec=t; tv_usec=0;
	 return *this;
}

inline UpTimeVal UpTimeVal::operator+(time_t a) const { // argument in seconds
	UpTimeVal c;
	c.tv_sec  = a + tv_sec;
	c.tv_usec = tv_usec;
	return c;
}

inline UpTimeVal UpTimeVal::operator-(time_t a) const { // argument in seconds
	UpTimeVal c;
	c.tv_sec  = tv_sec - a;
	c.tv_usec = tv_usec;
	return c;
}

inline bool UpTimeVal::operator>(time_t a) const { // argument in seconds
	return tv_sec>a || (tv_sec==a && tv_usec>0);
}

inline bool UpTimeVal::operator<(time_t a) const { // argument in seconds
	return tv_sec<a || (tv_sec==a && tv_usec<0);
}

inline bool UpTimeVal::operator==(time_t a) const { // argument in seconds
	return tv_sec==a && tv_usec==0;
}

inline bool UpTimeVal::operator!=(time_t a) const { // argument in seconds
	return tv_sec!=a || tv_usec != 0;
}

inline double UpTimeVal::operator/(time_t a) const { // argument in seconds
	return (1000000.0*(double)tv_sec + (double)tv_usec) / ((double)a*1000000.0);
}

inline UpTimeVal& UpTimeVal::operator+=(time_t a) { // argument in seconds
	tv_sec  = a + tv_sec;
	return *this;
}

inline UpTimeVal& UpTimeVal::operator-=(time_t a) {
	tv_sec  -= a;
	return *this;
}

/////////////////////////////////////////////////////////////////////////////
inline UpTimeVal& UpTimeVal::operator=(double d) { // argument in seconds
	register double fl=floor(d);
	tv_sec=(time_t)fl; tv_usec=(suseconds_t)(1000000.0*(d-fl));
	return *this;
}

inline UpTimeVal UpTimeVal::operator+(double d) const { // argument in seconds
	register double fl=floor(d);
	UpTimeVal c;
	c.tv_sec  = (time_t)fl  + tv_sec;
	c.tv_usec = (suseconds_t)(1000000.0*(d-fl)) + tv_usec;
	if( c.tv_usec > 999999) {
		c.tv_usec -= 1000000;
		(c.tv_sec)++;
	}
	return c;
}

inline UpTimeVal UpTimeVal::operator-(double d) const { // argument in seconds
	register double fl=floor(d);
	UpTimeVal c;
	c.tv_sec  = tv_sec  - (time_t)fl;
	c.tv_usec = tv_usec - (suseconds_t)((d-fl)*1000000.0);
	if( c.tv_usec < 0 ) {
		c.tv_usec += 1000000;
		(c.tv_sec)--; 
	}
	return c;
}

inline bool UpTimeVal::operator>(double d) const { // argument in seconds
	return ( ((double)tv_sec)*1000000.0 + (double)tv_usec ) > ( d*1000000.0 );
}

inline bool UpTimeVal::operator<(double d) const { // argument in seconds
	return ( ((double)tv_sec)*1000000.0 + (double)tv_usec ) < ( d*1000000.0 );
}

inline bool UpTimeVal::operator==(double d) const { // argument in seconds
	register double fl=floor(d);
	return tv_sec==(time_t)fl && tv_usec==(suseconds_t)(1000000.0*(d-fl));
}

inline bool UpTimeVal::operator!=(double d) const { // argument in seconds
	register double fl=floor(d);
	return tv_sec!=(time_t)fl || tv_usec!=(suseconds_t)(1000000.0*(d-fl));
}

inline double UpTimeVal::operator/(double d) const { // argument in seconds
	return ( ((double)tv_sec)*1000000.0 + (double)tv_usec ) / (d * 1000000.0);
}

inline UpTimeVal& UpTimeVal::operator+=(double d) { // argument in seconds
	register double fl=floor(d);
	tv_sec  = (time_t)fl + tv_sec;
	tv_usec = (suseconds_t)((d-fl)*1000000.0) + tv_usec;
	if( tv_usec > 999999) {
		tv_usec -= 1000000;
		tv_sec++;
	}
	return *this;
}

inline UpTimeVal& UpTimeVal::operator-=(double d) { // argument in seconds
	register double fl=floor(d);
	tv_sec  -= (time_t)fl;
	tv_usec -= (suseconds_t)((d-fl)*1000000.0);
	if( tv_usec < 0 ) {
		tv_usec += 1000000;
		--tv_sec; 
	}
	return *this;
}

/////////////////////////////////////////////////////////////////////////////
inline void UpTimeVal::copyTo(timeval& t) const { //copies itself to tv
	t.tv_sec=tv_sec;
	t.tv_usec=tv_usec;
}


inline timeval UpTimeVal::getStruct() const {
	return *((timeval*)this);
}

inline double UpTimeVal::getDouble() const {
	return ((double)tv_usec)/1000000.0 + (double)tv_sec;
} 

inline suseconds_t UpTimeVal::getTotalMicroSeconds(){
	return ((tv_sec*1000000)+tv_usec);
}

///////////////////////////////////////////////////////////////////////
inline UpTimeVal& UpTimeVal::operator*=(unsigned int n) {
	long long u1((long long)tv_usec);
	u1 *= n;
	long long u2(u1/1000000), u3(u1%1000000);
	tv_sec *= n;
	tv_sec += (time_t)u2;
	tv_usec = (time_t)u3;
	return *this;
}

inline UpTimeVal& UpTimeVal::operator*=(unsigned long long n) {
	long long u1((long long)tv_usec);
	u1 *= n;
	long long u2(u1/1000000), u3(u1%1000000);
	tv_sec *= n;
	tv_sec += (time_t)u2;
	tv_usec = (time_t)u3;
	return *this;
}

///////////////////////////////////////////////////////////////////////
inline UpTimeVal& UpTimeVal::operator*=(double x) {
	(*this) = x*getDouble();
	return *this;
}

